@@ -1,9 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
- * Copyright (C) 2009-2015,2017                           Université de Bordeaux
- * Copyright (C) 2010                                     Mehdi Juhoor
- * Copyright (C) 2010-2013,2015,2017                      CNRS
+ * Copyright (C) 2009-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010       Mehdi Juhoor
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -53,7 +51,7 @@ TYPE _alpha = 3.41;
 /* descriptors for StarPU */
 starpu_data_handle_t _handle_y, _handle_x;
 
-void axpy_cpu(void *descr[], STARPU_ATTRIBUTE_UNUSED void *arg)
+void axpy_cpu(void *descr[], void *arg)
 {
 	TYPE alpha = *((TYPE *)arg);
 
@@ -66,7 +64,7 @@ void axpy_cpu(void *descr[], STARPU_ATTRIBUTE_UNUSED void *arg)
 }
 
 #ifdef STARPU_USE_CUDA
-void axpy_gpu(void *descr[], STARPU_ATTRIBUTE_UNUSED void *arg)
+void axpy_gpu(void *descr[], void *arg)
 {
 	TYPE alpha = *((TYPE *)arg);
 
@@ -134,7 +132,7 @@ check(void)
 struct starpu_opencl_program opencl_program;
 #endif
 
-int main(int argc, char **argv)
+int main(void)
 {
 	int ret, exit_value = 0;
 
