@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010-2012, 2014  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013, 2016  CNRS
+ * Copyright (C) 2008-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -48,8 +48,7 @@ typedef struct
 } cl_args;
 
 #ifdef CHECK_RESULTS
-static void STARPU_ATTRIBUTE_UNUSED compare_A_LU(float *A, float *LU,
-				unsigned size, unsigned ld)
+static void compare_A_LU(float *A, float *LU, unsigned size, unsigned ld)
 {
 	unsigned i,j;
 	float *L;
@@ -215,4 +214,9 @@ void dw_callback_v2_codelet_update_u12(void *);
 void dw_callback_v2_codelet_update_u21(void *);
 void dw_callback_v2_codelet_update_u22(void *);
 
+extern struct starpu_perfmodel model_11;
+extern struct starpu_perfmodel model_12;
+extern struct starpu_perfmodel model_21;
+extern struct starpu_perfmodel model_22;
+
 #endif /* __DW_FACTO_LU_H__ */
