@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2013  Centre National de la Recherche Scientifique
+ * Copyright (C) 2009-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,6 +14,9 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
+/* This uses a dummy algorithm to downscale a ppm file. */
+/* TODO: turn this into StarPU. */
+
 #include "ppm_downscaler.h"
 
 #include <starpu.h>
@@ -51,7 +53,7 @@ struct ppm_image *file_to_ppm(char *filename)
 
 	struct ppm_image *ppm = (struct ppm_image *) malloc(sizeof(struct ppm_image));
 	assert(ppm);
-	
+
 	FILE *file = fopen(filename, "r");
 	assert(file);
 
@@ -63,7 +65,7 @@ struct ppm_image *file_to_ppm(char *filename)
 		fprintf(stderr, "file %s is not valid\n", filename);
 		exit(-1);
 	}
-	
+
 	/* allocate a buffer for the image */
 #ifdef STARPU_HAVE_MEMALIGN
 	ppm->data = (struct ppm_color *) memalign(16384, ppm->ncols*ppm->nlines*sizeof(struct ppm_color));
@@ -95,7 +97,7 @@ void ppm_to_file(struct ppm_image *ppm, char *filename)
 	fprintf(file, "P6\n%d %d\n%d\n", ppm->ncols, ppm->nlines, ppm->coldepth);
 
 	fwrite(&ppm->data[0], sizeof(struct ppm_color), ppm->ncols*ppm->nlines, file);
-	
+
 	fclose(file);
 }
 
@@ -130,7 +132,7 @@ void dummy_downscale(struct ppm_image *input_ppm, struct ppm_image *output_ppm)
 
 			unsigned big_col = col*FACTOR;
 			unsigned big_line = line*FACTOR;
-			
+
 			/* compute the average value of all components */
 			unsigned i, j;
 			for (i = 0; i < FACTOR; i++)
@@ -152,7 +154,7 @@ void dummy_downscale(struct ppm_image *input_ppm, struct ppm_image *output_ppm)
 			out[col + line*output_ppm->ncols].b = (unsigned char)(sum_b/(FACTOR*FACTOR));
 
 /*			fprintf(stderr, "col %d line %d -> sum_r = %d out -> %d\n", col, line, sum_r, out[col + line*FACTOR].r); */
-	
+
 		}
 	}
 }
@@ -177,5 +179,8 @@ int main(int argc, char **argv)
 
 	ppm_to_file(output_ppm, filename_out);
 
+	free(input_ppm);
+	free(output_ppm);
+
 	return 0;
 }
