@@ -1,9 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011,2013                                Inria
- * Copyright (C) 2009-2011,2013-2015,2017                 Université de Bordeaux
- * Copyright (C) 2010                                     Mehdi Juhoor
- * Copyright (C) 2010-2013,2015-2017                      CNRS
+ * Copyright (C) 2009-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010       Mehdi Juhoor
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -49,16 +47,17 @@ void parse_args(int argc, char **argv)
 	}
 	else
 	{
-		snprintf(filename_in, 1024, "%s/examples/ppm_downscaler/%s", STARPU_BUILD_DIR, filename_in_default);
-		snprintf(filename_out, 1024, "%s/examples/ppm_downscaler/%s", STARPU_BUILD_DIR, filename_out_default);
+		strncpy(filename_in, filename_in_default, 1023);
+		strncpy(filename_out, filename_out_default, 1023);
 	}
 }
 
 #define FRAMESIZE	sizeof(struct yuv_frame)
 #define NEW_FRAMESIZE	sizeof(struct yuv_new_frame)
 
-void ds_kernel_cpu(void *descr[], STARPU_ATTRIBUTE_UNUSED void *arg)
+void ds_kernel_cpu(void *descr[], void *arg)
 {
+	(void)arg;
 	uint8_t *input = (uint8_t *)STARPU_MATRIX_GET_PTR(descr[0]);
 	const unsigned input_ld = STARPU_MATRIX_GET_LD(descr[0]);
 
@@ -142,11 +141,19 @@ int main(int argc, char **argv)
 
 	/* fetch input data */
 	FILE *f_in = fopen(filename_in, "r");
-	assert(f_in);
+	if (!f_in)
+	{
+		fprintf(stderr, "couldn't open input file %s\n", filename_in);
+		exit(EXIT_FAILURE);
+	}
 
 	/* allocate room for an output buffer */
 	FILE *f_out = fopen(filename_out, "w+");
-	assert(f_out);
+	if (!f_out)
+	{
+		fprintf(stderr, "couldn't open output file %s\n", filename_out);
+		exit(EXIT_FAILURE);
+	}
 
 	sret = fread(yuv_in_buffer, FRAMESIZE, nframes, f_in);
 	assert(sret == nframes);
@@ -282,6 +289,13 @@ int main(int argc, char **argv)
 		starpu_data_unregister(new_frame_v_handle[frame]);
 	}
 
+	free(frame_y_handle);
+	free(frame_u_handle);
+	free(frame_v_handle);
+	free(new_frame_y_handle);
+	free(new_frame_u_handle);
+	free(new_frame_v_handle);
+
 	/* There is an implicit barrier: the unregister methods will block
 	 * until the computation is done and that the result was put back into
 	 * memory. */
