@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2013, 2015  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013  CNRS
+ * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -31,9 +30,10 @@ static unsigned niter = 50;
 static unsigned niter = 500;
 #endif
 
-void sleep_codelet(STARPU_ATTRIBUTE_UNUSED void *descr[],
-			STARPU_ATTRIBUTE_UNUSED void *_args)
+void sleep_codelet(void *descr[], void *arg)
 {
+	(void)descr;
+	(void)arg;
 	usleep(1000);
 }
 
@@ -118,8 +118,11 @@ int main(int argc, char **argv)
 
 	free(tasks);
 
-	FPRINTF(stderr, "Avg. delay : %2.2lf us\n", (delay_sum)/niter);
-	FPRINTF(stderr, "Avg. length : %2.2lf us\n", (length_sum)/niter);
+	if (niter)
+	{
+		FPRINTF(stderr, "Avg. delay : %2.2lf us\n", (delay_sum)/niter);
+		FPRINTF(stderr, "Avg. length : %2.2lf us\n", (length_sum)/niter);
+	}
 
 	/* Display the occupancy of all workers during the test */
 	unsigned worker;
@@ -141,6 +144,7 @@ int main(int argc, char **argv)
 		char workername[128];
 		starpu_worker_get_name(worker, workername, 128);
 		FPRINTF(stderr, "Worker %s:\n", workername);
+		FPRINTF(stderr, "\t%d task(s)\n", worker_info.executed_tasks);
 		FPRINTF(stderr, "\ttotal time : %.2lf ms\n", total_time*1e-3);
 		FPRINTF(stderr, "\texec time  : %.2lf ms (%.2f %%)\n", executing_time*1e-3, executing_ratio);
 		FPRINTF(stderr, "\tblocked time  : %.2lf ms (%.2f %%)\n", sleeping_time*1e-3, sleeping_ratio);
