@@ -1,8 +1,7 @@
 #!/bin/sh
 # StarPU --- Runtime system for heterogeneous multicore architectures.
 #
-# Copyright (C) 2012                                     Inria
-# Copyright (C) 2012-2015                                CNRS
+# Copyright (C) 2012-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
 #
 # StarPU is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -23,17 +22,23 @@ check_success()
     fi
 }
 
-if test ! -x ./cholesky/cholesky_tag
+basedir=$(dirname $0)
+if test ! -x $basedir/../cholesky/cholesky_tag
 then
-    echo "Application ./cholesky/cholesky_tag unavailable"
+    echo "Application $basedir/../cholesky/cholesky_tag unavailable"
     exit 77
 fi
 
-SCHEDULERS=`STARPU_SCHED="help" ./basic_examples/hello_world 2>&1 | awk '/\t->/ {print $1}'`
+if [ -n "$STARPU_SCHED" ]
+then
+	SCHEDULERS=$STARPU_SCHED
+else
+	SCHEDULERS=`$basedir/../../tools/starpu_sched_display | grep -v heteroprio`
+fi
 
 for sched in $SCHEDULERS
 do
     echo "cholesky.$sched"
-    STARPU_SCHED=$sched ./cholesky/cholesky_tag
+    STARPU_SCHED=$sched $STARPU_LAUNCH $basedir/../cholesky/cholesky_tag -size $((960*3)) -nblocks 3
     check_success $?
 done
