@@ -1,8 +1,7 @@
 #!/bin/sh
 # StarPU --- Runtime system for heterogeneous multicore architectures.
 #
-# Copyright (C) 2012,2014,2015,2017,2018                 CNRS
-# Copyright (C) 2017,2019                                Université de Bordeaux
+# Copyright (C) 2012-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
 #
 # StarPU is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -23,17 +22,23 @@ check_success()
     fi
 }
 
-if test ! -x ./sched_ctx/sched_ctx
+basedir=$(dirname $0)
+if test ! -x $basedir/../sched_ctx/sched_ctx
 then
-    echo "Application ./sched_ctx/sched_ctx unavailable"
+    echo "Application $basedir/../sched_ctx/sched_ctx unavailable"
     exit 77
 fi
 
-SCHEDULERS=`../tools/starpu_sched_display | grep -v pheft | grep -v peager | grep -v heteroprio | grep -v modular-gemm`
+if [ -n "$STARPU_SCHED" ]
+then
+	SCHEDULERS="$STARPU_SCHED"
+else
+	SCHEDULERS=`$basedir/../../tools/starpu_sched_display | grep -v pheft | grep -v peager | grep -v heteroprio | grep -v modular-gemm`
+fi
 
 for sched in $SCHEDULERS
 do
     echo "sched_ctx.$sched"
-    STARPU_SCHED=$sched ./sched_ctx/sched_ctx
+    STARPU_SCHED=$sched $STARPU_LAUNCH $basedir/../sched_ctx/sched_ctx
     check_success $?
 done
