@@ -1,10 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2014,2016                           Inria
- * Copyright (C) 2009-2019                                Université de Bordeaux
- * Copyright (C) 2010-2017, 2019                                CNRS
- * Copyright (C) 2013                                     Thibaut Lambert
- * Copyright (C) 2011                                     Télécom-SudParis
+ * Copyright (C) 2009-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011       Télécom-SudParis
+ * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -340,12 +338,12 @@ void starpu_perfmodel_get_model_path(const char *symbol, char *path, size_t maxl
 void starpu_perfmodel_dump_xml(FILE *output, struct starpu_perfmodel *model);
 
 /**
-   Free internal memory used for sampling directory
+   Free internal memory used for sampling
    management. It should only be called by an application which is not
    calling starpu_shutdown() as this function already calls it. See for
    example <c>tools/starpu_perfmodel_display.c</c>.
 */
-void starpu_perfmodel_free_sampling_directories(void);
+void starpu_perfmodel_free_sampling(void);
 
 /**
    Return the architecture type of the worker \p workerid.
