@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2015-2017                                CNRS
- * Copyright (C) 2015,2017                                Université de Bordeaux
+ * Copyright (C) 2015-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -26,7 +25,7 @@
 #define DATA0_TAG 12
 #define DATA1_TAG 22
 
-void func_cpu(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void func_cpu(void *descr[], void *_args)
 {
 	int *value = (int *)STARPU_VARIABLE_GET_PTR(descr[0]);
 	int rank;
@@ -40,7 +39,8 @@ struct starpu_codelet mycodelet =
 {
 	.cpu_funcs = {func_cpu},
 	.nbuffers = 1,
-	.modes = {STARPU_RW}
+	.modes = {STARPU_RW},
+	.model = &starpu_perfmodel_nop,
 };
 
 int main(int argc, char **argv)
@@ -89,10 +89,8 @@ int main(int argc, char **argv)
 		FPRINTF(stderr, "[%d][%d] received %d\n", rank, newrank, x);
 	}
 
-        ret = starpu_init(NULL);
-        STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
-        ret = starpu_mpi_init_comm(NULL, NULL, 0, newcomm);
-        STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init");
+        ret = starpu_mpi_init_conf(NULL, NULL, 0, newcomm, NULL);
+        STARPU_CHECK_RETURN_VALUE(ret, "starpu_mpi_init_conf");
 
 	if (newrank == 0)
 	{
@@ -147,7 +145,6 @@ int main(int argc, char **argv)
 	}
 
 	starpu_mpi_shutdown();
-	starpu_shutdown();
 	MPI_Comm_free(&newcomm);
         MPI_Finalize();
 	return 0;
