@@ -1,8 +1,6 @@
 ! StarPU --- Runtime system for heterogeneous multicore architectures.
 !
-! Copyright (C) 2017,2019                                CNRS
-! Copyright (C) 2016                                     Inria
-! Copyright (C) 2016,2017                                Université de Bordeaux
+! Copyright (C) 2016-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
 !
 ! StarPU is free software; you can redistribute it and/or modify
 ! it under the terms of the GNU Lesser General Public License as published by
@@ -30,7 +28,7 @@ module fstarpu_mpi_mod
                         type(c_ptr), value, intent(in) :: dh
                         type(c_ptr), value, intent(in) :: mpi_req
                         integer(c_int), value, intent(in) :: dst
-                        integer(c_int), value, intent(in) :: data_tag
+                        integer(c_int64_t), value, intent(in) :: data_tag
                         integer(c_int), value, intent(in) :: mpi_comm
                 end function fstarpu_mpi_isend
 
@@ -43,7 +41,7 @@ module fstarpu_mpi_mod
                         type(c_ptr), value, intent(in) :: dh
                         type(c_ptr), value, intent(in) :: mpi_req
                         integer(c_int), value, intent(in) :: dst
-                        integer(c_int), value, intent(in) :: data_tag
+                        integer(c_int64_t), value, intent(in) :: data_tag
                         integer(c_int), value, intent(in) :: prio
                         integer(c_int), value, intent(in) :: mpi_comm
                 end function fstarpu_mpi_isend_prio
@@ -56,7 +54,7 @@ module fstarpu_mpi_mod
                         type(c_ptr), value, intent(in) :: dh
                         type(c_ptr), value, intent(in) :: mpi_req
                         integer(c_int), value, intent(in) :: src
-                        integer(c_int), value, intent(in) :: data_tag
+                        integer(c_int64_t), value, intent(in) :: data_tag
                         integer(c_int), value, intent(in) :: mpi_comm
                 end function fstarpu_mpi_irecv
 
@@ -67,7 +65,7 @@ module fstarpu_mpi_mod
                         integer(c_int) :: fstarpu_mpi_send
                         type(c_ptr), value, intent(in) :: dh
                         integer(c_int), value, intent(in) :: dst
-                        integer(c_int), value, intent(in) :: data_tag
+                        integer(c_int64_t), value, intent(in) :: data_tag
                         integer(c_int), value, intent(in) :: mpi_comm
                 end function fstarpu_mpi_send
 
@@ -78,7 +76,7 @@ module fstarpu_mpi_mod
                         integer(c_int) :: fstarpu_mpi_send_prio
                         type(c_ptr), value, intent(in) :: dh
                         integer(c_int), value, intent(in) :: dst
-                        integer(c_int), value, intent(in) :: data_tag
+                        integer(c_int64_t), value, intent(in) :: data_tag
                         integer(c_int), value, intent(in) :: prio
                         integer(c_int), value, intent(in) :: mpi_comm
                 end function fstarpu_mpi_send_prio
@@ -90,7 +88,7 @@ module fstarpu_mpi_mod
                         integer(c_int) :: fstarpu_mpi_recv
                         type(c_ptr), value, intent(in) :: dh
                         integer(c_int), value, intent(in) :: src
-                        integer(c_int), value, intent(in) :: data_tag
+                        integer(c_int64_t), value, intent(in) :: data_tag
                         integer(c_int), value, intent(in) :: mpi_comm
                         type(c_ptr), value, intent(in) :: mpi_status
                 end function fstarpu_mpi_recv
@@ -102,7 +100,7 @@ module fstarpu_mpi_mod
                         integer(c_int) :: fstarpu_mpi_isend_detached
                         type(c_ptr), value, intent(in) :: dh
                         integer(c_int), value, intent(in) :: dst
-                        integer(c_int), value, intent(in) :: data_tag
+                        integer(c_int64_t), value, intent(in) :: data_tag
                         integer(c_int), value, intent(in) :: mpi_comm
                         type(c_funptr), value, intent(in) :: callback
                         type(c_ptr), value, intent(in) :: arg
@@ -115,7 +113,7 @@ module fstarpu_mpi_mod
                         integer(c_int) :: fstarpu_mpi_isend_detached_prio
                         type(c_ptr), value, intent(in) :: dh
                         integer(c_int), value, intent(in) :: dst
-                        integer(c_int), value, intent(in) :: data_tag
+                        integer(c_int64_t), value, intent(in) :: data_tag
                         integer(c_int), value, intent(in) :: prio
                         integer(c_int), value, intent(in) :: mpi_comm
                         type(c_funptr), value, intent(in) :: callback
@@ -129,7 +127,7 @@ module fstarpu_mpi_mod
                         integer(c_int) :: fstarpu_mpi_recv_detached
                         type(c_ptr), value, intent(in) :: dh
                         integer(c_int), value, intent(in) :: src
-                        integer(c_int), value, intent(in) :: data_tag
+                        integer(c_int64_t), value, intent(in) :: data_tag
                         integer(c_int), value, intent(in) :: mpi_comm
                         type(c_funptr), value, intent(in) :: callback
                         type(c_ptr), value, intent(in) :: arg
@@ -143,7 +141,7 @@ module fstarpu_mpi_mod
                         type(c_ptr), value, intent(in) :: dh
                         type(c_ptr), value, intent(in) :: mpi_req
                         integer(c_int), value, intent(in) :: dst
-                        integer(c_int), value, intent(in) :: data_tag
+                        integer(c_int64_t), value, intent(in) :: data_tag
                         integer(c_int), value, intent(in) :: mpi_comm
                 end function fstarpu_mpi_issend
 
@@ -155,7 +153,7 @@ module fstarpu_mpi_mod
                         type(c_ptr), value, intent(in) :: dh
                         type(c_ptr), value, intent(in) :: mpi_req
                         integer(c_int), value, intent(in) :: dst
-                        integer(c_int), value, intent(in) :: data_tag
+                        integer(c_int64_t), value, intent(in) :: data_tag
                         integer(c_int), value, intent(in) :: prio
                         integer(c_int), value, intent(in) :: mpi_comm
                 end function fstarpu_mpi_issend_prio
@@ -167,7 +165,7 @@ module fstarpu_mpi_mod
                         integer(c_int) :: fstarpu_mpi_issend_detached
                         type(c_ptr), value, intent(in) :: dh
                         integer(c_int), value, intent(in) :: dst
-                        integer(c_int), value, intent(in) :: data_tag
+                        integer(c_int64_t), value, intent(in) :: data_tag
                         integer(c_int), value, intent(in) :: mpi_comm
                         type(c_funptr), value, intent(in) :: callback
                         type(c_ptr), value, intent(in) :: arg
@@ -180,7 +178,7 @@ module fstarpu_mpi_mod
                         integer(c_int) :: fstarpu_mpi_issend_detached_prio
                         type(c_ptr), value, intent(in) :: dh
                         integer(c_int), value, intent(in) :: dst
-                        integer(c_int), value, intent(in) :: data_tag
+                        integer(c_int64_t), value, intent(in) :: data_tag
                         integer(c_int), value, intent(in) :: prio
                         integer(c_int), value, intent(in) :: mpi_comm
                         type(c_funptr), value, intent(in) :: callback
@@ -222,7 +220,7 @@ module fstarpu_mpi_mod
                         integer(c_int) :: fstarpu_mpi_recv_detached_sequential_consistency
                         type(c_ptr), value, intent(in) :: dh
                         integer(c_int), value, intent(in) :: src
-                        integer(c_int), value, intent(in) :: data_tag
+                        integer(c_int64_t), value, intent(in) :: data_tag
                         integer(c_int), value, intent(in) :: mpi_comm
                         type(c_funptr), value, intent(in) :: callback
                         type(c_ptr), value, intent(in) :: arg
@@ -344,7 +342,7 @@ module fstarpu_mpi_mod
                         integer(c_int) :: fstarpu_mpi_isend_detached_unlock_tag
                         type(c_ptr), value, intent(in) :: dh
                         integer(c_int), value, intent(in) :: dst
-                        integer(c_int), value, intent(in) :: data_tag
+                        integer(c_int64_t), value, intent(in) :: data_tag
                         integer(c_int), value, intent(in) :: mpi_comm
                         type(c_ptr), value, intent(in) :: starpu_tag
                 end function fstarpu_mpi_isend_detached_unlock_tag
@@ -356,7 +354,7 @@ module fstarpu_mpi_mod
                         integer(c_int) :: fstarpu_mpi_isend_detached_unlock_tag_prio
                         type(c_ptr), value, intent(in) :: dh
                         integer(c_int), value, intent(in) :: dst
-                        integer(c_int), value, intent(in) :: data_tag
+                        integer(c_int64_t), value, intent(in) :: data_tag
                         integer(c_int), value, intent(in) :: prio
                         integer(c_int), value, intent(in) :: mpi_comm
                         type(c_ptr), value, intent(in) :: starpu_tag
@@ -369,7 +367,7 @@ module fstarpu_mpi_mod
                         integer(c_int) :: fstarpu_mpi_recv_detached_unlock_tag
                         type(c_ptr), value, intent(in) :: dh
                         integer(c_int), value, intent(in) :: src
-                        integer(c_int), value, intent(in) :: data_tag
+                        integer(c_int64_t), value, intent(in) :: data_tag
                         integer(c_int), value, intent(in) :: mpi_comm
                         type(c_ptr), value, intent(in) :: starpu_tag
                 end function fstarpu_mpi_recv_detached_unlock_tag
@@ -383,7 +381,7 @@ module fstarpu_mpi_mod
                         integer(c_int), value, intent(in) :: array_size
                         type(c_ptr), intent(in) :: dhs(*)
                         integer(c_int), intent(in) :: dsts(*)
-                        integer(c_int), intent(in) :: data_tags(*)
+                        integer(c_int64_t), intent(in) :: data_tags(*)
                         integer(c_int), intent(in) :: mpi_comms(*)
                         type(c_ptr), value, intent(in) :: starpu_tag
                 end function fstarpu_mpi_isend_array_detached_unlock_tag
@@ -397,7 +395,7 @@ module fstarpu_mpi_mod
                         integer(c_int), value, intent(in) :: array_size
                         type(c_ptr), intent(in) :: dhs(*)
                         integer(c_int), intent(in) :: dsts(*)
-                        integer(c_int), intent(in) :: data_tags(*)
+                        integer(c_int64_t), intent(in) :: data_tags(*)
                         integer(c_int), intent(in) :: prio(*)
                         integer(c_int), intent(in) :: mpi_comms(*)
                         type(c_ptr), value, intent(in) :: starpu_tag
@@ -412,7 +410,7 @@ module fstarpu_mpi_mod
                         integer(c_int), value, intent(in) :: array_size
                         type(c_ptr), intent(in) :: dhs(*)
                         integer(c_int), intent(in) :: srcs(*)
-                        integer(c_int), intent(in) :: data_tags(*)
+                        integer(c_int64_t), intent(in) :: data_tags(*)
                         integer(c_int), intent(in) :: mpi_comms(*)
                         type(c_ptr), value, intent(in) :: starpu_tag
                 end function fstarpu_mpi_recv_array_detached_unlock_tag
@@ -491,7 +489,7 @@ module fstarpu_mpi_mod
                 subroutine fstarpu_mpi_set_communication_tag(tag) bind(C,name="starpu_mpi_set_communication_tag")
                         use iso_c_binding
                         implicit none
-                        integer(c_int), value, intent(in) :: tag
+                        integer(c_int64_t), value, intent(in) :: tag
                 end subroutine fstarpu_mpi_set_communication_tag
 
                 ! void starpu_mpi_data_register_comm(starpu_data_handle_t data_handle, int tag, int rank, MPI_Comm comm);
@@ -499,7 +497,7 @@ module fstarpu_mpi_mod
                         use iso_c_binding
                         implicit none
                         type(c_ptr), value, intent(in) :: dh
-                        integer(c_int), value, intent(in) :: tag
+                        integer(c_int64_t), value, intent(in) :: tag
                         integer(c_int), value, intent(in) :: rank
                         integer(c_int), value, intent(in) :: mpi_comm
                 end subroutine fstarpu_mpi_data_register_comm
@@ -509,7 +507,7 @@ module fstarpu_mpi_mod
                         use iso_c_binding
                         implicit none
                         type(c_ptr), value, intent(in) :: dh
-                        integer(c_int), value, intent(in) :: tag
+                        integer(c_int64_t), value, intent(in) :: tag
                         integer(c_int), value, intent(in) :: rank
                 end subroutine fstarpu_mpi_data_register
 
@@ -535,7 +533,7 @@ module fstarpu_mpi_mod
                         use iso_c_binding
                         implicit none
                         type(c_ptr), value, intent(in) :: dh
-                        integer(c_int), value, intent(in) :: tag
+                        integer(c_int64_t), value, intent(in) :: tag
                 end subroutine fstarpu_mpi_data_set_tag
 
                 ! int starpu_mpi_data_get_rank(starpu_data_handle_t handle);
@@ -550,7 +548,7 @@ module fstarpu_mpi_mod
                 function fstarpu_mpi_data_get_tag(dh) bind(C,name="starpu_mpi_data_get_tag")
                         use iso_c_binding
                         implicit none
-                        integer(c_int) :: fstarpu_mpi_data_get_tag
+                        integer(c_int64_t) :: fstarpu_mpi_data_get_tag
                         type(c_ptr), value, intent(in) :: dh
                 end function fstarpu_mpi_data_get_tag
 
