@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2013                                     Inria
- * Copyright (C) 2011-2017                                CNRS
- * Copyright (C) 2012-2015,2017,2018                      Université de Bordeaux
+ * Copyright (C) 2011-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -39,7 +37,7 @@ struct starpu_codelet mycodelet =
 {
 	.cpu_funcs = {func_cpu},
 	.nbuffers = 4,
-	.modes = {STARPU_R, STARPU_RW, STARPU_W, STARPU_W},
+	.modes = {STARPU_R, STARPU_RW, STARPU_W, STARPU_RW},
 	.model = &starpu_perfmodel_nop,
 };
 
@@ -96,7 +94,7 @@ int main(int argc, char **argv)
 	err = starpu_mpi_task_insert(MPI_COMM_WORLD, &mycodelet,
 				     STARPU_R, data_handles[0], STARPU_RW, data_handles[1],
 				     STARPU_W, data_handles[2],
-				     STARPU_W, data_handles[3],
+				     STARPU_RW, data_handles[3],
 				     STARPU_EXECUTE_ON_NODE, 1, 0);
 	STARPU_CHECK_RETURN_VALUE(err, "starpu_mpi_task_insert");
 	starpu_task_wait_for_all();
