@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2014                                Inria
- * Copyright (C) 2015-2017                                CNRS
- * Copyright (C) 2013-2014,2017                           Université de Bordeaux
+ * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -252,6 +250,8 @@ double sc_hypervisor_get_avg_speed(enum starpu_worker_archtype arch)
 		if(max_real_start_time < sc_w->real_start_time)
 			max_real_start_time = sc_w->real_start_time;
 	}
+	free(sched_ctxs);
+
 	double speed = -1.0;
 #ifdef STARPU_SC_HYPERVISOR_DEBUG
 	printf("total_exec_flops %lf total_estimated_flops %lf max_real_start_time %lf nworkers %u \n", total_executed_flops, total_estimated_flops, max_real_start_time, nworkers);
@@ -284,9 +284,7 @@ void _consider_max_for_children(unsigned sched_ctx, unsigned consider_max)
 	int s;
 	for(s = 0; s < nsched_ctxs_child; s++)
 		_consider_max_for_children(sched_ctxs_child[s], consider_max);
-	if(nsched_ctxs_child > 0)
-		free(sched_ctxs_child);
-	return;
+	free(sched_ctxs_child);
 }
 
 void sc_hypervisor_check_if_consider_max(struct types_of_workers *tw)
@@ -324,4 +322,5 @@ void sc_hypervisor_check_if_consider_max(struct types_of_workers *tw)
 				_consider_max_for_children(sched_ctxs[s], 0);
 		}
 	}
+	free(sched_ctxs);
 }
