@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011                                     Inria
- * Copyright (C) 2012,2015,2017                           CNRS
- * Copyright (C) 2010-2011, 2018                                Université de Bordeaux
+ * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -27,19 +25,20 @@ soclGetMemObjectInfo(cl_mem           mem,
 		     void *           param_value,
 		     size_t *         param_value_size_ret)
 {
-   static cl_mem_object_type mot = CL_MEM_OBJECT_BUFFER;
+	static cl_mem_object_type mot = CL_MEM_OBJECT_BUFFER;
 
-   switch (param_name) {
-      INFO_CASE(CL_MEM_TYPE, mot)
-      INFO_CASE(CL_MEM_FLAGS, mem->flags)
-      INFO_CASE(CL_MEM_SIZE, mem->size)
-      INFO_CASE(CL_MEM_HOST_PTR, mem->host_ptr)
-      INFO_CASE(CL_MEM_MAP_COUNT, mem->map_count)
-      INFO_CASE(CL_MEM_REFERENCE_COUNT, mem->_entity.refs)
-      INFO_CASE(CL_MEM_CONTEXT, mem->context)
-      default:
-         return CL_INVALID_VALUE;
-   }
+	switch (param_name)
+	{
+		INFO_CASE(CL_MEM_TYPE, mot);
+		INFO_CASE(CL_MEM_FLAGS, mem->flags);
+		INFO_CASE(CL_MEM_SIZE, mem->size);
+		INFO_CASE(CL_MEM_HOST_PTR, mem->host_ptr);
+		INFO_CASE(CL_MEM_MAP_COUNT, mem->map_count);
+		INFO_CASE(CL_MEM_REFERENCE_COUNT, mem->_entity.refs);
+		INFO_CASE(CL_MEM_CONTEXT, mem->context);
+	default:
+		return CL_INVALID_VALUE;
+	}
 
-   return CL_SUCCESS;
+	return CL_SUCCESS;
 }
