@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010,2011 University of Bordeaux
+ * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,10 +16,11 @@
 
 #include "socl.h"
 
+CL_API_SUFFIX__VERSION_1_0
 CL_API_ENTRY cl_int CL_API_CALL
-soclReleaseMemObject(cl_mem mem) CL_API_SUFFIX__VERSION_1_0
+soclReleaseMemObject(cl_mem mem)
 {
-  gc_entity_release(mem);
+	gc_entity_release(mem);
 
-  return CL_SUCCESS;
+	return CL_SUCCESS;
 }
