@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012                                     Inria
- * Copyright (C) 2012,2014-2015,2017                      CNRS
- * Copyright (C) 2010-2013                                Université de Bordeaux
+ * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -29,7 +27,7 @@ extern volatile int _starpu_init;
  * Initialize StarPU
  */
 
-void socl_init_starpu(void);
+int socl_init_starpu(void);
 void soclShutdown(void);
 
 #endif /* SOCL_INIT_H */
