@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010  Université de Bordeaux 1
- * Copyright (C) 2010  Centre National de la Recherche Scientifique
+ * Copyright (C) 2008-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -18,20 +17,22 @@
 #ifndef TIMING_H
 #define TIMING_H
 
-/*
+/** @file */
+
+#include <stdint.h>
+#include <common/config.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#include <starpu.h>
+#include <starpu_util.h>
+
+/**
  * _starpu_timing_init must be called prior to using any of these timing
  * functions.
  */
-
-#include <sys/time.h>
-#include <unistd.h>
-#include <stdint.h>
-#include <common/config.h>
-#include <starpu.h>
-
 void _starpu_timing_init(void);
-void starpu_clock_gettime(struct timespec *ts);
-double starpu_timing_now(void);
+void _starpu_clock_gettime(struct timespec *ts);
 
 #endif /* TIMING_H */
 
