@@ -24,6 +24,8 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #ifndef UTHASH_H
 #define UTHASH_H 
 
+/** @file */
+
 #include <string.h>   /* memcmp,strlen */
 #include <stddef.h>   /* ptrdiff_t */
 
@@ -81,7 +83,7 @@ typedef unsigned int uint32_t;
 
 #define HASH_FIND(hh,head,keyptr,keylen,out)                                     \
 do {                                                                             \
-  unsigned _hf_bkt,_hf_hashv;                                                    \
+  unsigned _hf_bkt=0,_hf_hashv=0;					         \
   out=NULL;                                                                      \
   if (head) {                                                                    \
      HASH_FCN(keyptr,keylen, (head)->hh.tbl->num_buckets, _hf_hashv, _hf_bkt);   \
@@ -153,7 +155,7 @@ do {
 do {                                                                             \
   __typeof__(out) _out;                                                          \
   HASH_FIND(hh,head,keyptr,keylen,_out);                                         \
-  STARPU_ASSERT(!_out);                                                          \
+  STARPU_ASSERT_MSG(!_out,"Cannot insert the same key twice"); 			 \
 } while(0)
 #else
 #define HASH_CHECK_KEY(hh,head,keyptr,keylen,out)
@@ -161,7 +163,7 @@ do {
 
 #define HASH_ADD_KEYPTR(hh,head,keyptr,keylen_in,add)                            \
 do {                                                                             \
- unsigned _ha_bkt;                                                               \
+ unsigned _ha_bkt=0;							 	 \
  HASH_CHECK_KEY(hh,head,keyptr,keylen_in,add);                                   \
  (add)->hh.next = NULL;                                                          \
  (add)->hh.key = (char*)keyptr;                                                  \
@@ -444,6 +446,7 @@ do {
      /* FALLTHRU */                                                              \
      case 1:  _hj_i += _hj_key[0];                                               \
      /* FALLTHRU */                                                              \
+     default: break;                                                             \
   }                                                                              \
   HASH_JEN_MIX(_hj_i, _hj_j, hashv);                                             \
   bkt = hashv & (num_bkts-1);                                                    \
@@ -492,6 +495,8 @@ do {
     case 1: hashv += *_sfh_key;                                                  \
             hashv ^= hashv << 10;                                                \
             hashv += hashv >> 1;                                                 \
+            break;                                                               \
+    default: break;                                                              \
   }                                                                              \
                                                                                  \
     /* Force "avalanching" of final 127 bits */                                  \
@@ -543,9 +548,13 @@ do {
   switch(_mur_len)                                                               \
   {                                                                              \
     case 3: hashv ^= _mur_key[2] << 16;                                          \
+    /* FALLTHRU */                                                               \
     case 2: hashv ^= _mur_key[1] << 8;                                           \
+    /* FALLTHRU */                                                               \
     case 1: hashv ^= _mur_key[0];                                                \
             hashv *= _mur_m;                                                     \
+    /* FALLTHRU */                                                               \
+    default: break;                                                              \
   };                                                                             \
                                                                                  \
   hashv ^= hashv >> 13;                                                          \
@@ -569,8 +578,12 @@ do {
     unsigned _mur_t = 0, _mur_d = 0;                                             \
     switch(_mur_align) {                                                         \
       case 1: _mur_t |= _mur_key[2] << 16;                                       \
+      /* FALLTHRU */                                                             \
       case 2: _mur_t |= _mur_key[1] << 8;                                        \
+      /* FALLTHRU */                                                             \
       case 3: _mur_t |= _mur_key[0];                                             \
+      /* FALLTHRU */                                                             \
+      default: break;                                                            \
     }                                                                            \
     _mur_t <<= (8 * _mur_align);                                                 \
     _mur_key += 4-_mur_align;                                                    \
@@ -594,8 +607,12 @@ do {
     if(_mur_len >= _mur_align) {                                                 \
       switch(_mur_align) {                                                       \
         case 3: _mur_d |= _mur_key[2] << 16;                                     \
+        /* FALLTHRU */                                                           \
         case 2: _mur_d |= _mur_key[1] << 8;                                      \
+        /* FALLTHRU */                                                           \
         case 1: _mur_d |= _mur_key[0];                                           \
+        /* FALLTHRU */                                                           \
+        default: break;                                                          \
       }                                                                          \
       unsigned _mur_k = (_mur_t >> _mur_sr) | (_mur_d << _mur_sl);               \
       _mur_k *= _mur_m;                                                          \
@@ -609,18 +626,27 @@ do {
       switch(_mur_len)                                                           \
       {                                                                          \
         case 3: hashv ^= _mur_key[2] << 16;                                      \
+        /* FALLTHRU */                                                           \
         case 2: hashv ^= _mur_key[1] << 8;                                       \
+        /* FALLTHRU */                                                           \
         case 1: hashv ^= _mur_key[0];                                            \
                 hashv *= _mur_m;                                                 \
+        /* FALLTHRU */                                                           \
+        default: break;                                                          \
       }                                                                          \
     } else {                                                                     \
       switch(_mur_len)                                                           \
       {                                                                          \
         case 3: _mur_d ^= _mur_key[2] << 16;                                     \
+        /* FALLTHRU */                                                           \
         case 2: _mur_d ^= _mur_key[1] << 8;                                      \
+        /* FALLTHRU */                                                           \
         case 1: _mur_d ^= _mur_key[0];                                           \
+        /* FALLTHRU */                                                           \
         case 0: hashv ^= (_mur_t >> _mur_sr) | (_mur_d << _mur_sl);              \
         hashv *= _mur_m;                                                         \
+        /* FALLTHRU */                                                           \
+        default: break;                                                          \
       }                                                                          \
     }                                                                            \
                                                                                  \
@@ -640,9 +666,13 @@ do {
     switch(_mur_len)                                                             \
     {                                                                            \
       case 3: hashv ^= _mur_key[2] << 16;                                        \
+      /* FALLTHRU */                                                             \
       case 2: hashv ^= _mur_key[1] << 8;                                         \
+      /* FALLTHRU */                                                             \
       case 1: hashv ^= _mur_key[0];                                              \
       hashv *= _mur_m;                                                           \
+      /* FALLTHRU */                                                             \
+      default: break;                                                            \
     }                                                                            \
                                                                                  \
     hashv ^= hashv >> 13;                                                        \
