@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010, 2012-2017  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013, 2014, 2015, 2017  CNRS
+ * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -24,6 +23,7 @@
 #include <unistd.h>
 #endif
 #include <fcntl.h>
+#include <ctype.h>
 
 #if defined(_WIN32) && !defined(__CYGWIN__)
 #include <io.h>
@@ -78,9 +78,11 @@ int _starpu_mkpath(const char *s, mode_t mode)
 {
 	int olderrno;
 	char *q, *r = NULL, *path = NULL, *up = NULL;
-	int rv;
+	int rv = -1;
+
+	while (s[0] == '/' && s[1] == '/')
+		s++;
 
-	rv = -1;
 	if (strcmp(s, ".") == 0 || strcmp(s, "/") == 0
 #if defined(_WIN32)
 		/* C:/ or C:\ */
@@ -109,7 +111,7 @@ int _starpu_mkpath(const char *s, mode_t mode)
 	{
 		if (!S_ISDIR(sb.st_mode))
 		{
-			_STARPU_MSG("Error: %s is not a directory:\n", path);
+			_STARPU_MSG("Error: %s already exists and is not a directory:\n", path);
 			STARPU_ABORT();
 		}
 		/* It already exists and is a directory.  */
@@ -201,7 +203,8 @@ char *_starpu_mktemp(const char *directory, int flags, int *fd)
 {
 	/* create template for mkstemp */
 	const char *tmp = "STARPU_XXXXXX";
-	char *baseCpy = malloc(strlen(directory)+1+strlen(tmp)+1);
+	char *baseCpy;
+	_STARPU_MALLOC(baseCpy, strlen(directory)+1+strlen(tmp)+1);
 
 	snprintf(baseCpy, strlen(directory)+1+strlen(tmp)+1, "%s/%s", directory, tmp);
 
@@ -226,12 +229,13 @@ char *_starpu_mktemp(const char *directory, int flags, int *fd)
 	if (*fd < 0)
 	{
 		int err = errno;
-		_STARPU_DISP("Could not create temporary file in directory '%s', mk[o]stemp failed with error '%s'\n", directory, strerror(errno));
+		if (err != ENOENT)
+			_STARPU_DISP("Could not create temporary file in directory '%s', mk[o]stemp failed with error '%s'\n", directory, strerror(errno));
 		free(baseCpy);
 		errno = err;
 		return NULL;
 	}
-	
+
 #if !defined(STARPU_HAVE_WINDOWS) && !defined (HAVE_MKOSTEMP)
 	/* Add O_DIRECT after the mkstemp call */
 	if ((flags & O_DIRECT) != 0)
@@ -247,10 +251,11 @@ char *_starpu_mktemp(const char *directory, int flags, int *fd)
 			free(baseCpy);
 			errno = err;
 			return NULL;
-		}		
+		}
 	}
 #endif
 
+
 	return baseCpy;
 }
 
@@ -274,6 +279,7 @@ char *_starpu_mktemp_many(const char *directory, int depth, int flags, int *fd)
 	}
 
 	memcpy(path, directory, len+1);
+retry:
 	for (i = 0; i < depth; i++)
 	{
 		int r = starpu_lrand48();
@@ -305,6 +311,11 @@ char *_starpu_mktemp_many(const char *directory, int depth, int flags, int *fd)
 	retpath = _starpu_mktemp(path, flags, fd);
 	if (!retpath)
 	{
+		if (errno == ENOENT)
+		{
+			/* Somebody else dropped our directory, retry */
+			goto retry;
+		}
 		/* That failed, drop our directories */
 		_starpu_rmdir_many(path, depth);
 	}
@@ -327,7 +338,7 @@ void _starpu_rmdir_many(char *path, int depth)
 	int i;
 	for (i = 0; i < depth; i++)
 	{
-		if (rmdir(path) < 0 && errno != ENOTEMPTY && errno != EBUSY)
+		if (rmdir(path) < 0 && errno != ENOTEMPTY && errno != EBUSY && errno != ENOENT)
 			_STARPU_DISP("Could not remove temporary directory '%s', rmdir failed with error '%s'\n", path, strerror(errno));
 		path = dirname(path);
 	}
@@ -452,6 +463,7 @@ int _starpu_fwrlock(FILE *file)
 
 int _starpu_fwrunlock(FILE *file)
 {
+	fflush(file);
 	return _starpu_frdunlock(file);
 }
 
@@ -527,7 +539,11 @@ void _starpu_gethostname(char *hostname, size_t size)
 void starpu_sleep(float nb_sec)
 {
 #ifdef STARPU_SIMGRID
+#  ifdef HAVE_SG_ACTOR_SLEEP_FOR
+	sg_actor_sleep_for(nb_sec);
+#  else
 	MSG_process_sleep(nb_sec);
+#  endif
 #elif defined(STARPU_HAVE_WINDOWS)
 	Sleep(nb_sec * 1000);
 #else
@@ -540,6 +556,23 @@ void starpu_sleep(float nb_sec)
 #endif
 }
 
+void starpu_usleep(float nb_micro_sec)
+{
+#ifdef STARPU_SIMGRID
+#  ifdef HAVE_SG_ACTOR_SLEEP_FOR
+	sg_actor_sleep_for(nb_micro_sec / 1000000);
+#  else
+	MSG_process_sleep(nb_micro_sec / 1000000);
+#  endif
+#elif defined(STARPU_HAVE_WINDOWS)
+	Sleep(nb_micro_sec / 1000);
+#elif HAVE_UNISTD_H
+	usleep(nb_micro_sec);
+#else
+#error no implementation of usleep
+#endif
+}
+
 char *starpu_getenv(const char *str)
 {
 #ifndef STARPU_SIMGRID
@@ -554,3 +587,136 @@ char *starpu_getenv(const char *str)
 #endif
 	return getenv(str);
 }
+
+int _strings_ncmp(const char *strings[], const char *str)
+{
+	int pos = 0;
+	while (strings[pos])
+	{
+		if ((strlen(str) == strlen(strings[pos]) && strncasecmp(str, strings[pos], strlen(strings[pos])) == 0))
+			break;
+		pos++;
+	}
+	if (strings[pos] == NULL)
+		return -1;
+	return pos;
+}
+
+int starpu_get_env_string_var_default(const char *str, const char *strings[], int defvalue)
+{
+	int val;
+	char *strval;
+
+	strval = starpu_getenv(str);
+	if (!strval)
+	{
+		val = defvalue;
+	}
+	else
+	{
+		val = _strings_ncmp(strings, strval);
+		if (val < 0)
+		{
+			int i;
+			_STARPU_MSG("\n");
+			_STARPU_MSG("Invalid value '%s' for environment variable '%s'\n", strval, str);
+			_STARPU_MSG("Valid values are:\n");
+			for(i=0;strings[i]!=NULL;i++) _STARPU_MSG("\t%s\n",strings[i]);
+			_STARPU_MSG("\n");
+			STARPU_ABORT();
+		}
+	}
+	return val;
+}
+
+static void remove_spaces(char *str)
+{
+	int i = 0;
+	int j = 0;
+
+	while (str[j] != '\0')
+	{
+		if (isspace(str[j]))
+		{
+			j++;
+			continue;
+		}
+		if (j > i)
+		{
+			str[i] = str[j];
+		}
+		i++;
+		j++;
+	}
+	if (j > i)
+	{
+		str[i] = str[j];
+	}
+}
+
+int starpu_get_env_size_default(const char *str, int defval)
+{
+	int val;
+	char *strval;
+
+	strval = starpu_getenv(str);
+	if (!strval)
+	{
+		val = defval;
+	}
+	else
+	{
+		char *value = strdup(strval);
+		if (value == NULL)
+			_STARPU_ERROR("memory allocation failed\n");
+		remove_spaces(value);
+		if (value[0] == '\0')
+		{
+			free(value);
+			val = defval;
+		}
+		else
+		{
+			char *endptr = NULL;
+			int mult = 1024;
+			errno = 0;
+			int v = (int)strtol(value, &endptr, 10);
+			if (errno != 0)
+				_STARPU_ERROR("could not parse environment variable '%s' with value '%s', strtol failed with error %s\n", str, value, strerror(errno));
+			if (*endptr != '\0')
+			{
+				switch (*endptr)
+				{
+				case 'b':
+				case 'B': mult = 1; break;
+				case 'k':
+				case 'K': mult = 1024; break;
+				case 'm':
+				case 'M': mult = 1024*1024; break;
+				case 'g':
+				case 'G': mult = 1024*1024*1024; break;
+				default:
+					_STARPU_ERROR("could not parse environment variable '%s' with value '%s' size suffix invalid\n", str, value);
+				}
+			}
+			val = v*mult;
+			free(value);
+		}
+	}
+	return val;
+}
+
+void starpu_display_bindings(void)
+{
+#ifdef STARPU_HAVE_HWLOC
+	int hwloc_ret = system("hwloc-ps -a -t -c");
+	if (hwloc_ret)
+	{
+		_STARPU_DISP("hwloc-ps returned %d\n", hwloc_ret);
+		fflush(stderr);
+	}
+	fflush(stdout);
+#else
+	_STARPU_DISP("hwloc not available to display bindings.\n");
+#endif
+}
