@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2018                                Université de Bordeaux
- * Copyright (C) 2010,2011,2013,2015,2017                 CNRS
+ * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -18,10 +17,12 @@
 #ifndef __CG_H__
 #define __CG_H__
 
+/** @file */
+
 #include <starpu.h>
 #include <common/config.h>
 
-/* we do not necessarily want to allocate room for 256 dependencies, but we
+/** we do not necessarily want to allocate room for 256 dependencies, but we
    want to handle the few situation where there are a lot of dependencies as
    well */
 #define STARPU_DYNAMIC_DEPS_SIZE	1
@@ -33,33 +34,34 @@
 
 struct _starpu_job;
 
-/* Completion Group list, records both the number of expected notifications
+/** Completion Group list, records both the number of expected notifications
  * before the completion can start, and the list of successors when the
  * completion is finished. */
 struct _starpu_cg_list
 {
-	/* Protects atomicity of the list and the terminated flag */
+	/** Protects atomicity of the list and the terminated flag */
 	struct _starpu_spinlock lock;
 
-	/* Number of notifications to be waited for */
+	/** Number of notifications to be waited for */
 	unsigned ndeps; /* how many deps ? */
 	unsigned ndeps_completed; /* how many deps are done ? */
 #ifdef STARPU_DEBUG
-	/* Array of the notifications, size ndeps */
+	/** Array of the notifications, size ndeps */
 	struct _starpu_cg **deps;
-	/* Which ones have notified, size ndeps */
+	/** Which ones have notified, size ndeps */
 	char *done;
 #endif
 
-	/* Whether the completion is finished.
+	/** Whether the completion is finished.
 	 * For restartable/restarted tasks, only the first iteration is taken into account here.
 	 */
 	unsigned terminated;
 
-	/* List of successors */
+	/** List of successors */
 	unsigned nsuccs; /* how many successors ? */
 #ifdef STARPU_DYNAMIC_DEPS_SIZE
-	unsigned succ_list_size; /* How many allocated items in succ */
+	/** How many allocated items in succ */
+	unsigned succ_list_size;
 	struct _starpu_cg **succ;
 #else
 	struct _starpu_cg *succ[STARPU_NMAXDEPS];
@@ -73,30 +75,34 @@ enum _starpu_cg_type
 	STARPU_CG_TASK=(1<<2)
 };
 
-/* Completion Group */
+/** Completion Group */
 struct _starpu_cg
 {
-	unsigned ntags; /* number of tags depended on */
-	unsigned remaining; /* number of remaining tags */
+	/** number of tags depended on */
+	unsigned ntags;
+	/** number of remaining tags */
+	unsigned remaining;
 
 #ifdef STARPU_DEBUG
 	unsigned ndeps;
-	void **deps; /* array of predecessors, size ndeps */
-	char *done;  /* which ones have notified, size ndeps */
+	/** array of predecessors, size ndeps */
+	void **deps;
+	/** which ones have notified, size ndeps */
+	char *done;
 #endif
 
 	enum _starpu_cg_type cg_type;
 
 	union
 	{
-		/* STARPU_CG_TAG */
+		/** STARPU_CG_TAG */
 		struct _starpu_tag *tag;
 
-		/* STARPU_CG_TASK */
+		/** STARPU_CG_TASK */
 		struct _starpu_job *job;
 
-		/* STARPU_CG_APPS */
-		/* in case this completion group is related to an application,
+		/** STARPU_CG_APPS
+		 * in case this completion group is related to an application,
 		 * we have to explicitely wake the waiting thread instead of
 		 * reschedule the corresponding task */
 		struct
