@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012                                     Inria
- * Copyright (C) 2008-2011,2014                           Université de Bordeaux
- * Copyright (C) 2010-2011,2015                           CNRS
+ * Copyright (C) 2008-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,6 +17,8 @@
 #ifndef __TAGS_H__
 #define __TAGS_H__
 
+/** @file */
+
 #include <starpu.h>
 #include <common/config.h>
 #include <common/starpu_spinlock.h>
@@ -28,19 +28,18 @@
 
 enum _starpu_tag_state
 {
-	/* this tag is not declared by any task */
+	/** this tag is not declared by any task */
 	STARPU_INVALID_STATE,
-	/* _starpu_tag_declare was called to associate the tag to a task */
+	/** _starpu_tag_declare was called to associate the tag to a task */
 	STARPU_ASSOCIATED,
-	/* some task dependencies are not fulfilled yet */
+	/** some task dependencies are not fulfilled yet */
 	STARPU_BLOCKED,
-	/* the task can be (or has been) submitted to the scheduler (all deps
- 	 * fulfilled) */
+	/** the task can be (or has been) submitted to the scheduler (all deps fulfilled) */
 	STARPU_READY,
 // useless ...
-//	/* the task has been submitted to the scheduler */
+//	/** the task has been submitted to the scheduler */
 //	STARPU_SCHEDULED,
-	/* the task has been performed */
+	/** the task has been performed */
 	STARPU_DONE
 };
 
@@ -48,15 +47,18 @@ struct _starpu_job;
 
 struct _starpu_tag
 {
-	/* Lock for this structure. Locking order is in dependency order: a tag
-	 * must not be locked before locking a tag it depends on */
+	/**
+	   Lock for this structure. Locking order is in dependency order: a tag
+	   * must not be locked before locking a tag it depends on */
 	struct _starpu_spinlock lock;
-	starpu_tag_t id; /* an identifier for the task */
+	/** an identifier for the task */
+	starpu_tag_t id;
 	enum _starpu_tag_state state;
 
 	struct _starpu_cg_list tag_successors;
 
-	struct _starpu_job *job; /* which job is associated to the tag if any ? */
+	/** which job is associated to the tag if any ? */
+	struct _starpu_job *job;
 
 	unsigned is_assigned;
 	unsigned is_submitted;
@@ -64,10 +66,12 @@ struct _starpu_tag
 
 void _starpu_init_tags(void);
 
-void _starpu_notify_dependencies(struct _starpu_job *j);
 void _starpu_notify_tag_dependencies(struct _starpu_tag *tag);
+void _starpu_notify_job_start_tag_dependencies(struct _starpu_tag *tag, _starpu_notify_job_start_data *data);
 
 void _starpu_tag_declare(starpu_tag_t id, struct _starpu_job *job);
+
+/** lock should be taken, and this releases it */
 void _starpu_tag_set_ready(struct _starpu_tag *tag);
 
 unsigned _starpu_submit_job_enforce_task_deps(struct _starpu_job *j);
