@@ -1,11 +1,9 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2017                                Inria
- * Copyright (C) 2013                                     Simon Archipoff
- * Copyright (C) 2008-2019                                Université de Bordeaux
- * Copyright (C) 2010-2017, 2019                          CNRS
- * Copyright (C) 2013                                     Thibaut Lambert
- * Copyright (C) 2016                                     Uppsala University
+ * Copyright (C) 2008-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013       Simon Archipoff
+ * Copyright (C) 2013       Thibaut Lambert
+ * Copyright (C) 2016       Uppsala University
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -445,9 +443,9 @@ int _starpu_repush_task(struct _starpu_job *j)
 	{
 		/*if there are workers in the ctx that are not able to execute tasks
 		  we consider the ctx empty */
-		unsigned nworkers = _starpu_nworkers_able_to_execute_task(task, sched_ctx);
+		unsigned able = _starpu_workers_able_to_execute_task(task, sched_ctx);
 
-		if(nworkers == 0)
+		if(!able)
 		{
 			_starpu_sched_ctx_lock_write(sched_ctx->id);
 			starpu_task_list_push_front(&sched_ctx->empty_ctx_tasks, task);
@@ -506,7 +504,6 @@ int _starpu_repush_task(struct _starpu_job *j)
 int _starpu_push_task_to_workers(struct starpu_task *task)
 {
 	struct _starpu_sched_ctx *sched_ctx = _starpu_get_sched_ctx_struct(task->sched_ctx);
-	unsigned nworkers = 0;
 
 	_STARPU_TRACE_JOB_PUSH(task, task->priority > 0);
 
@@ -516,9 +513,9 @@ int _starpu_push_task_to_workers(struct starpu_task *task)
 	{
 		/*if there are workers in the ctx that are not able to execute tasks
 		  we consider the ctx empty */
-		nworkers = _starpu_nworkers_able_to_execute_task(task, sched_ctx);
+		unsigned able = _starpu_workers_able_to_execute_task(task, sched_ctx);
 
-		if (nworkers == 0)
+		if (!able)
 		{
 			_starpu_sched_ctx_lock_write(sched_ctx->id);
 			starpu_task_list_push_back(&sched_ctx->empty_ctx_tasks, task);
@@ -554,7 +551,7 @@ int _starpu_push_task_to_workers(struct starpu_task *task)
 		/* When a task can only be executed on a given arch and we have
 		 * only one memory node for that arch, we can systematically
 		 * prefetch before the scheduling decision. */
-		if (starpu_get_prefetch_flag())
+		if (starpu_get_prefetch_flag() && starpu_memory_nodes_get_count() > 1)
 		{
 			if (task->where == STARPU_CPU && config->cpus_nodeid >= 0)
 				starpu_prefetch_task_input_on_node(task, config->cpus_nodeid);
@@ -610,7 +607,7 @@ int _starpu_push_task_to_workers(struct starpu_task *task)
 		{
 			STARPU_ASSERT(sched_ctx->sched_policy->push_task);
 			/* check out if there are any workers in the context */
-			nworkers = starpu_sched_ctx_get_nworkers(sched_ctx->id);
+			unsigned nworkers = starpu_sched_ctx_get_nworkers(sched_ctx->id);
 			if (nworkers == 0)
 				ret = -1;
 			else
