@@ -1,11 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011                                     Antoine Lucas
- * Copyright (C) 2011-2012,2016                           Inria
- * Copyright (C) 2008-2017                                Université de Bordeaux
- * Copyright (C) 2010                                     Mehdi Juhoor
- * Copyright (C) 2010-2013,2015-2017                      CNRS
- * Copyright (C) 2013                                     Thibaut Lambert
+ * Copyright (C) 2008-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010       Mehdi Juhoor
+ * Copyright (C) 2013       Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,6 +16,8 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
+//#define STARPU_VERBOSE
+
 #include <datawizard/filters.h>
 #include <datawizard/footprint.h>
 #include <datawizard/interfaces/data_interface.h>
@@ -175,6 +174,7 @@ static void _starpu_data_partition(starpu_data_handle_t initial_handle, starpu_d
 	if (inherit_state)
 	{
 		_STARPU_CALLOC(initial_handle->children, nparts, sizeof(struct _starpu_data_state));
+
 		/* this handle now has children */
 		initial_handle->nchildren = nparts;
 	}
@@ -189,14 +189,18 @@ static void _starpu_data_partition(starpu_data_handle_t initial_handle, starpu_d
 		/* This is lazy allocation, allocate it now in main RAM, so as
 		 * to have somewhere to gather pieces later */
 		/* FIXME: mark as unevictable! */
-		int ret = _starpu_allocate_memory_on_node(initial_handle, &initial_handle->per_node[STARPU_MAIN_RAM], 0);
+		int home_node = initial_handle->home_node;
+		if (home_node < 0 || (starpu_node_get_kind(home_node) != STARPU_CPU_RAM))
+			home_node = STARPU_MAIN_RAM;
+		int ret = _starpu_allocate_memory_on_node(initial_handle, &initial_handle->per_node[home_node], STARPU_FETCH);
 #ifdef STARPU_DEVEL
 #warning we should reclaim memory if allocation failed
 #endif
 		STARPU_ASSERT(!ret);
 	}
 
-	_starpu_data_unregister_ram_pointer(initial_handle);
+	for (node = 0; node < STARPU_MAXNODES; node++)
+		_starpu_data_unregister_ram_pointer(initial_handle, node);
 
 	if (nparts && !inherit_state)
 	{
@@ -229,9 +233,19 @@ static void _starpu_data_partition(starpu_data_handle_t initial_handle, starpu_d
 		child->switch_cl = NULL;
 		child->partitioned = 0;
 		child->readonly = 0;
-                child->mpi_data = initial_handle->mpi_data;
+		child->active = inherit_state;
+		child->active_ro = 0;
+                child->mpi_data = NULL;
 		child->root_handle = initial_handle->root_handle;
 		child->father_handle = initial_handle;
+		child->active_children = NULL;
+		child->active_readonly_children = NULL;
+		child->nactive_readonly_children = 0;
+		child->nsiblings = nparts;
+		if (inherit_state)
+			child->siblings = NULL;
+		else
+			child->siblings = childrenp;
 		child->sibling_index = i;
 		child->depth = initial_handle->depth + 1;
 
@@ -240,8 +254,8 @@ static void _starpu_data_partition(starpu_data_handle_t initial_handle, starpu_d
 		child->home_node = initial_handle->home_node;
 
 		/* initialize the chunk lock */
-		_starpu_data_requester_list_init(&child->req_list);
-		_starpu_data_requester_list_init(&child->reduction_req_list);
+		_starpu_data_requester_prio_list_init(&child->req_list);
+		_starpu_data_requester_prio_list_init(&child->reduction_req_list);
 		child->reduction_tmp_handles = NULL;
 		child->write_invalidation_req = NULL;
 		child->refcnt = 0;
@@ -255,6 +269,7 @@ static void _starpu_data_partition(starpu_data_handle_t initial_handle, starpu_d
 
 		child->sequential_consistency = initial_handle->sequential_consistency;
 		child->initialized = initial_handle->initialized;
+		child->ooc = initial_handle->ooc;
 
 		STARPU_PTHREAD_MUTEX_INIT(&child->sequential_consistency_mutex, NULL);
 		child->last_submitted_mode = STARPU_R;
@@ -283,7 +298,7 @@ static void _starpu_data_partition(starpu_data_handle_t initial_handle, starpu_d
 			starpu_data_assign_arbiter(child, _starpu_global_arbiter);
 		else
 			child->arbiter = NULL;
-		_starpu_data_requester_list_init(&child->arbitered_req_list);
+		_starpu_data_requester_prio_list_init(&child->arbitered_req_list);
 
 		for (node = 0; node < STARPU_MAXNODES; node++)
 		{
@@ -326,10 +341,14 @@ static void _starpu_data_partition(starpu_data_handle_t initial_handle, starpu_d
 		 * store it in the handle */
 		child->footprint = _starpu_compute_data_footprint(child);
 
-		void *ptr;
-		ptr = starpu_data_handle_to_pointer(child, STARPU_MAIN_RAM);
-		if (ptr != NULL)
-			_starpu_data_register_ram_pointer(child, ptr);
+		for (node = 0; node < STARPU_MAXNODES; node++)
+		{
+			if (starpu_node_get_kind(node) != STARPU_CPU_RAM)
+				continue;
+			void *ptr = starpu_data_handle_to_pointer(child, node);
+			if (ptr != NULL)
+				_starpu_data_register_ram_pointer(child, ptr);
+		}
 
 		_STARPU_TRACE_HANDLE_DATA_REGISTER(child);
 	}
@@ -423,25 +442,38 @@ void starpu_data_unpartition(starpu_data_handle_t root_handle, unsigned gatherin
 
 		_starpu_spin_lock(&child_handle->header_lock);
 
-		sizes[child] = _starpu_data_get_size(child_handle);
+		sizes[child] = _starpu_data_get_alloc_size(child_handle);
 
-		_starpu_data_unregister_ram_pointer(child_handle);
+		if (child_handle->unregister_hook)
+		{
+			child_handle->unregister_hook(child_handle);
+		}
+
+		for (node = 0; node < STARPU_MAXNODES; node++)
+			_starpu_data_unregister_ram_pointer(child_handle, node);
 
 		if (child_handle->per_worker)
-		for (worker = 0; worker < nworkers; worker++)
 		{
-			struct _starpu_data_replicate *local = &child_handle->per_worker[worker];
-			STARPU_ASSERT(local->state == STARPU_INVALID);
-			if (local->allocated && local->automatically_allocated)
-				_starpu_request_mem_chunk_removal(child_handle, local, starpu_worker_get_memory_node(worker), sizes[child]);
+			for (worker = 0; worker < nworkers; worker++)
+			{
+				struct _starpu_data_replicate *local = &child_handle->per_worker[worker];
+				STARPU_ASSERT(local->state == STARPU_INVALID);
+				if (local->allocated && local->automatically_allocated)
+					_starpu_request_mem_chunk_removal(child_handle, local, starpu_worker_get_memory_node(worker), sizes[child]);
+			}
 		}
 
 		_starpu_memory_stats_free(child_handle);
 	}
 
-	ptr = starpu_data_handle_to_pointer(root_handle, STARPU_MAIN_RAM);
-	if (ptr != NULL)
-		_starpu_data_register_ram_pointer(root_handle, ptr);
+	for (node = 0; node < STARPU_MAXNODES; node++)
+	{
+		if (starpu_node_get_kind(node) != STARPU_CPU_RAM)
+			continue;
+		ptr = starpu_data_handle_to_pointer(root_handle, node);
+		if (ptr != NULL)
+			_starpu_data_register_ram_pointer(root_handle, ptr);
+	}
 
 	/* the gathering_node should now have a valid copy of all the children.
 	 * For all nodes, if the node had all copies and none was locally
@@ -489,7 +521,7 @@ void starpu_data_unpartition(starpu_data_handle_t root_handle, unsigned gatherin
 
 		if (!isvalid && local->mc && local->allocated && local->automatically_allocated)
 			/* free the data copy in a lazy fashion */
-			_starpu_request_mem_chunk_removal(root_handle, local, node, _starpu_data_get_size(root_handle));
+			_starpu_request_mem_chunk_removal(root_handle, local, node, _starpu_data_get_alloc_size(root_handle));
 
 		/* if there was no invalid copy, the node still has a valid copy */
 		still_valid[node] = isvalid;
@@ -504,8 +536,7 @@ void starpu_data_unpartition(starpu_data_handle_t root_handle, unsigned gatherin
 
 	for (node = 0; node < STARPU_MAXNODES; node++)
 	{
-		root_handle->per_node[node].state =
-			still_valid[node]?newstate:STARPU_INVALID;
+		root_handle->per_node[node].state = still_valid[node]?newstate:STARPU_INVALID;
 	}
 
 	for (child = 0; child < root_handle->nchildren; child++)
@@ -516,6 +547,25 @@ void starpu_data_unpartition(starpu_data_handle_t root_handle, unsigned gatherin
 		_starpu_spin_destroy(&child_handle->header_lock);
 	}
 
+	/* Set the initialized state */
+	starpu_data_handle_t first_child = starpu_data_get_child(root_handle, 0);
+	root_handle->initialized = first_child->initialized;
+	for (child = 1; child < root_handle->nchildren; child++)
+	{
+		starpu_data_handle_t child_handle = starpu_data_get_child(root_handle, child);
+		STARPU_ASSERT_MSG(child_handle->initialized == root_handle->initialized, "Inconsistent state between children initialization");
+	}
+	if (root_handle->initialized)
+	{
+		for (node = 0; node < STARPU_MAXNODES; node++)
+		{
+			struct _starpu_data_replicate *root_replicate;
+
+			root_replicate = &root_handle->per_node[node];
+			root_replicate->initialized = still_valid[node];
+		}
+	}
+
 	for (child = 0; child < root_handle->nchildren; child++)
 	{
 		starpu_data_handle_t child_handle = starpu_data_get_child(root_handle, child);
@@ -564,6 +614,7 @@ void starpu_data_partition_plan(starpu_data_handle_t initial_handle, struct star
 	STARPU_ASSERT_MSG(initial_handle->sequential_consistency, "partition planning is currently only supported for data with sequential consistency");
 	struct starpu_codelet *cl = initial_handle->switch_cl;
 	int home_node = initial_handle->home_node;
+	starpu_data_handle_t *children;
 	if (home_node == -1)
 		/* Nothing better for now */
 		/* TODO: pass -1, and make _starpu_fetch_nowhere_task_input
@@ -572,9 +623,13 @@ void starpu_data_partition_plan(starpu_data_handle_t initial_handle, struct star
 		 */
 		home_node = STARPU_MAIN_RAM;
 
+	_STARPU_MALLOC(children, nparts * sizeof(*children));
 	for (i = 0; i < nparts; i++)
-		_STARPU_CALLOC(childrenp[i], 1, sizeof(struct _starpu_data_state));
-	_starpu_data_partition(initial_handle, childrenp, nparts, f, 0);
+	{
+		_STARPU_CALLOC(children[i], 1, sizeof(struct _starpu_data_state));
+		childrenp[i] = children[i];
+	}
+	_starpu_data_partition(initial_handle, children, nparts, f, 0);
 
 	if (!cl)
 	{
@@ -583,6 +638,7 @@ void starpu_data_partition_plan(starpu_data_handle_t initial_handle, struct star
 		cl = initial_handle->switch_cl;
 		cl->where = STARPU_NOWHERE;
 		cl->nbuffers = STARPU_VARIABLE_NBUFFERS;
+		cl->flags = STARPU_CODELET_NOPLANS;
 		cl->name = "data_partition_switch";
 		cl->specific_nodes = 1;
 	}
@@ -600,6 +656,16 @@ void starpu_data_partition_clean(starpu_data_handle_t root_handle, unsigned npar
 {
 	unsigned i;
 
+	if (children[0]->active)
+	{
+#ifdef STARPU_DEVEL
+#warning FIXME: better choose gathering node
+#endif
+		starpu_data_unpartition_submit(root_handle, nparts, children, STARPU_MAIN_RAM);
+	}
+
+	free(children[0]->siblings);
+
 	for (i = 0; i < nparts; i++)
 		starpu_data_unregister_submit(children[i]);
 
@@ -608,46 +674,98 @@ void starpu_data_partition_clean(starpu_data_handle_t root_handle, unsigned npar
 	_starpu_spin_unlock(&root_handle->header_lock);
 }
 
-void starpu_data_partition_submit(starpu_data_handle_t initial_handle, unsigned nparts, starpu_data_handle_t *children)
+static
+void _starpu_data_partition_submit(starpu_data_handle_t initial_handle, unsigned nparts, starpu_data_handle_t *children, unsigned char *handles_sequential_consistency)
 {
+	unsigned i;
 	STARPU_ASSERT_MSG(initial_handle->sequential_consistency, "partition planning is currently only supported for data with sequential consistency");
 	_starpu_spin_lock(&initial_handle->header_lock);
 	STARPU_ASSERT_MSG(initial_handle->partitioned == 0, "One can't submit several partition plannings at the same time");
 	STARPU_ASSERT_MSG(initial_handle->readonly == 0, "One can't submit a partition planning while a readonly partitioning is active");
+	STARPU_ASSERT_MSG(nparts > 0, "One can't partition into 0 parts");
 	initial_handle->partitioned++;
+	initial_handle->active_children = children[0]->siblings;
 	_starpu_spin_unlock(&initial_handle->header_lock);
 
+	for (i = 0; i < nparts; i++)
+	{
+		_starpu_spin_lock(&children[i]->header_lock);
+		children[i]->active = 1;
+		_starpu_spin_unlock(&children[i]->header_lock);
+	}
+
 	if (!initial_handle->initialized)
 		/* No need for coherency, it is not initialized */
 		return;
-	unsigned i;
+
 	struct starpu_data_descr descr[nparts];
 	for (i = 0; i < nparts; i++)
 	{
-		STARPU_ASSERT_MSG(children[i]->father_handle == initial_handle, "children parameter of starpu_data_partition_submit must be the children of the parent parameter");
+		STARPU_ASSERT_MSG(children[i]->father_handle == initial_handle, "child(%d) %p is partitioned from %p and not from the given parameter %p", i, children[i], children[i]->father_handle, initial_handle);
 		descr[i].handle = children[i];
 		descr[i].mode = STARPU_W;
 	}
 	/* TODO: assert nparts too */
-	starpu_task_insert(initial_handle->switch_cl, STARPU_RW, initial_handle, STARPU_DATA_MODE_ARRAY, descr, nparts, 0);
-	starpu_data_invalidate_submit(initial_handle);
+	int ret;
+	if (handles_sequential_consistency)
+		ret = starpu_task_insert(initial_handle->switch_cl, STARPU_RW, initial_handle, STARPU_DATA_MODE_ARRAY, descr, nparts,
+					 STARPU_NAME, "partition",
+					 STARPU_HANDLES_SEQUENTIAL_CONSISTENCY, handles_sequential_consistency,
+					 0);
+	else
+		ret = starpu_task_insert(initial_handle->switch_cl, STARPU_RW, initial_handle, STARPU_DATA_MODE_ARRAY, descr, nparts,
+					 STARPU_NAME, "partition",
+					 0);
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
+	if (!handles_sequential_consistency || handles_sequential_consistency[0])
+		starpu_data_invalidate_submit(initial_handle);
+}
+
+void starpu_data_partition_submit_sequential_consistency(starpu_data_handle_t initial_handle, unsigned nparts, starpu_data_handle_t *children, int sequential_consistency)
+{
+	unsigned i;
+	unsigned char handles_sequential_consistency[nparts+1];
+	handles_sequential_consistency[0] = sequential_consistency;
+	for(i=1 ; i<nparts+1 ; i++) handles_sequential_consistency[i] = children[i-1]->sequential_consistency;
+
+	_starpu_data_partition_submit(initial_handle, nparts, children, handles_sequential_consistency);
+}
+
+void starpu_data_partition_submit(starpu_data_handle_t initial_handle, unsigned nparts, starpu_data_handle_t *children)
+{
+	_starpu_data_partition_submit(initial_handle, nparts, children, NULL);
 }
 
 void starpu_data_partition_readonly_submit(starpu_data_handle_t initial_handle, unsigned nparts, starpu_data_handle_t *children)
 {
+	unsigned i;
 	STARPU_ASSERT_MSG(initial_handle->sequential_consistency, "partition planning is currently only supported for data with sequential consistency");
 	_starpu_spin_lock(&initial_handle->header_lock);
 	STARPU_ASSERT_MSG(initial_handle->partitioned == 0 || initial_handle->readonly, "One can't submit a readonly partition planning at the same time as a readwrite partition planning");
+	STARPU_ASSERT_MSG(nparts > 0, "One can't partition into 0 parts");
 	initial_handle->partitioned++;
 	initial_handle->readonly = 1;
+	if (initial_handle->nactive_readonly_children < initial_handle->partitioned)
+	{
+		_STARPU_REALLOC(initial_handle->active_readonly_children, initial_handle->partitioned * sizeof(initial_handle->active_readonly_children[0]));
+		initial_handle->nactive_readonly_children = initial_handle->partitioned;
+	}
+	initial_handle->active_readonly_children[initial_handle->partitioned-1] = children[0]->siblings;
 	_starpu_spin_unlock(&initial_handle->header_lock);
 
+	for (i = 0; i < nparts; i++)
+	{
+		_starpu_spin_lock(&children[i]->header_lock);
+		children[i]->active = 1;
+		children[i]->active_ro = 1;
+		_starpu_spin_unlock(&children[i]->header_lock);
+	}
+
 	STARPU_ASSERT_MSG(initial_handle->initialized, "It is odd to read-only-partition a data which does not have a value yet");
-	unsigned i;
 	struct starpu_data_descr descr[nparts];
 	for (i = 0; i < nparts; i++)
 	{
-		STARPU_ASSERT_MSG(children[i]->father_handle == initial_handle, "children parameter of starpu_data_partition_submit must be the children of the parent parameter");
+		STARPU_ASSERT_MSG(children[i]->father_handle == initial_handle, "child(%d) %p is partitioned from %p and not from the given parameter %p", i, children[i], children[i]->father_handle, initial_handle);
 		descr[i].handle = children[i];
 		descr[i].mode = STARPU_W;
 	}
@@ -661,14 +779,18 @@ void starpu_data_partition_readwrite_upgrade_submit(starpu_data_handle_t initial
 	_starpu_spin_lock(&initial_handle->header_lock);
 	STARPU_ASSERT_MSG(initial_handle->partitioned == 1, "One can't upgrade a readonly partition planning to readwrite while other readonly partition plannings are active");
 	STARPU_ASSERT_MSG(initial_handle->readonly == 1, "One can only upgrade a readonly partition planning");
+	STARPU_ASSERT_MSG(nparts > 0, "One can't partition into 0 parts");
 	initial_handle->readonly = 0;
+	initial_handle->active_children = initial_handle->active_readonly_children[0];
+	initial_handle->active_readonly_children[0] = NULL;
 	_starpu_spin_unlock(&initial_handle->header_lock);
 
 	unsigned i;
 	struct starpu_data_descr descr[nparts];
 	for (i = 0; i < nparts; i++)
 	{
-		STARPU_ASSERT_MSG(children[i]->father_handle == initial_handle, "children parameter of starpu_data_partition_submit must be the children of the parent parameter");
+		STARPU_ASSERT_MSG(children[i]->father_handle == initial_handle, "child(%d) %p is partitioned from %p and not from the given parameter %p", i, children[i], children[i]->father_handle, initial_handle);
+		children[i]->active_ro = 0;
 		descr[i].handle = children[i];
 		descr[i].mode = STARPU_W;
 	}
@@ -677,22 +799,50 @@ void starpu_data_partition_readwrite_upgrade_submit(starpu_data_handle_t initial
 	starpu_data_invalidate_submit(initial_handle);
 }
 
-void starpu_data_unpartition_submit(starpu_data_handle_t initial_handle, unsigned nparts, starpu_data_handle_t *children, int gather_node)
+void _starpu_data_unpartition_submit(starpu_data_handle_t initial_handle, unsigned nparts, starpu_data_handle_t *children, int gather_node, unsigned char *handles_sequential_consistency, void (*callback_func)(void *), void *callback_arg)
 {
+	unsigned i;
 	STARPU_ASSERT_MSG(initial_handle->sequential_consistency, "partition planning is currently only supported for data with sequential consistency");
 	STARPU_ASSERT_MSG(gather_node == initial_handle->home_node || gather_node == -1, "gathering node different from home node is currently not supported");
 	_starpu_spin_lock(&initial_handle->header_lock);
-	STARPU_ASSERT_MSG(initial_handle->partitioned >= 1, "No partition planning is active for this handle");
+	STARPU_ASSERT_MSG(initial_handle->partitioned >= 1, "No partition planning is active for handle %p", initial_handle);
+	STARPU_ASSERT_MSG(nparts > 0, "One can't partition into 0 parts");
+	if (initial_handle->readonly)
+	{
+		/* Replace this children set with the last set in the list of readonly children sets */
+		for (i = 0; i < initial_handle->partitioned-1; i++)
+		{
+			if (initial_handle->active_readonly_children[i] == children[0]->siblings)
+			{
+				initial_handle->active_readonly_children[i] = initial_handle->active_readonly_children[initial_handle->partitioned-1];
+				initial_handle->active_readonly_children[initial_handle->partitioned-1] = NULL;
+				break;
+			}
+		}
+	}
+	else
+	{
+		initial_handle->active_children = NULL;
+	}
 	initial_handle->partitioned--;
 	if (!initial_handle->partitioned)
 		initial_handle->readonly = 0;
+	initial_handle->active_children = NULL;
 	_starpu_spin_unlock(&initial_handle->header_lock);
 
-	unsigned i, n;
+	for (i = 0; i < nparts; i++)
+	{
+		_starpu_spin_lock(&children[i]->header_lock);
+		children[i]->active = 0;
+		children[i]->active_ro = 0;
+		_starpu_spin_unlock(&children[i]->header_lock);
+	}
+
+	unsigned n;
 	struct starpu_data_descr descr[nparts];
 	for (i = 0, n = 0; i < nparts; i++)
 	{
-		STARPU_ASSERT_MSG(children[i]->father_handle == initial_handle, "children parameter of starpu_data_partition_submit must be the children of the parent parameter");
+		STARPU_ASSERT_MSG(children[i]->father_handle == initial_handle, "child(%d) %p is partitioned from %p and not from the given parameter %p", i, children[i], children[i]->father_handle, initial_handle);
 		if (!children[i]->initialized)
 			/* Dropped value, do not care about coherency for this one */
 			continue;
@@ -701,9 +851,48 @@ void starpu_data_unpartition_submit(starpu_data_handle_t initial_handle, unsigne
 		n++;
 	}
 	/* TODO: assert nparts too */
-	starpu_task_insert(initial_handle->switch_cl, STARPU_W, initial_handle, STARPU_DATA_MODE_ARRAY, descr, n, 0);
+	int ret;
+	if (handles_sequential_consistency)
+		ret = starpu_task_insert(initial_handle->switch_cl, STARPU_W, initial_handle, STARPU_DATA_MODE_ARRAY, descr, n,
+					 STARPU_NAME, "unpartition",
+					 STARPU_HANDLES_SEQUENTIAL_CONSISTENCY, handles_sequential_consistency,
+					 STARPU_CALLBACK_WITH_ARG_NFREE, callback_func, callback_arg,
+					 0);
+	else
+		ret = starpu_task_insert(initial_handle->switch_cl, STARPU_W, initial_handle, STARPU_DATA_MODE_ARRAY, descr, n,
+					 STARPU_NAME, "unpartition",
+					 STARPU_CALLBACK_WITH_ARG_NFREE, callback_func, callback_arg,
+					 0);
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
+
 	for (i = 0; i < nparts; i++)
-		starpu_data_invalidate_submit(children[i]);
+	{
+		if (!handles_sequential_consistency || handles_sequential_consistency[i+1])
+			starpu_data_invalidate_submit(children[i]);
+	}
+}
+
+void starpu_data_unpartition_submit(starpu_data_handle_t initial_handle, unsigned nparts, starpu_data_handle_t *children, int gather_node)
+{
+	_starpu_data_unpartition_submit(initial_handle, nparts, children, gather_node, NULL, NULL, NULL);
+}
+
+void starpu_data_unpartition_submit_sequential_consistency_cb(starpu_data_handle_t initial_handle, unsigned nparts, starpu_data_handle_t *children, int gather_node, int sequential_consistency, void (*callback_func)(void *), void *callback_arg)
+{
+	unsigned i;
+	unsigned char handles_sequential_consistency[nparts+1];
+	handles_sequential_consistency[0] = sequential_consistency;
+	for(i=1 ; i<nparts+1 ; i++) handles_sequential_consistency[i] = children[i-1]->sequential_consistency;
+	_starpu_data_unpartition_submit(initial_handle, nparts, children, gather_node, handles_sequential_consistency, callback_func, callback_arg);
+}
+
+void starpu_data_unpartition_submit_sequential_consistency(starpu_data_handle_t initial_handle, unsigned nparts, starpu_data_handle_t *children, int gather_node, int sequential_consistency)
+{
+	unsigned i;
+	unsigned char handles_sequential_consistency[nparts+1];
+	handles_sequential_consistency[0] = sequential_consistency;
+	for(i=1 ; i<nparts+1 ; i++) handles_sequential_consistency[i] = children[i-1]->sequential_consistency;
+	_starpu_data_unpartition_submit(initial_handle, nparts, children, gather_node, handles_sequential_consistency, NULL, NULL);
 }
 
 void starpu_data_unpartition_readonly_submit(starpu_data_handle_t initial_handle, unsigned nparts, starpu_data_handle_t *children, int gather_node)
@@ -711,7 +900,8 @@ void starpu_data_unpartition_readonly_submit(starpu_data_handle_t initial_handle
 	STARPU_ASSERT_MSG(initial_handle->sequential_consistency, "partition planning is currently only supported for data with sequential consistency");
 	STARPU_ASSERT_MSG(gather_node == initial_handle->home_node || gather_node == -1, "gathering node different from home node is currently not supported");
 	_starpu_spin_lock(&initial_handle->header_lock);
-	STARPU_ASSERT_MSG(initial_handle->partitioned >= 1, "No partition planning is active for this handle");
+	STARPU_ASSERT_MSG(initial_handle->partitioned >= 1, "No partition planning is active for handle %p", initial_handle);
+	STARPU_ASSERT_MSG(nparts > 0, "One can't partition into 0 parts");
 	initial_handle->readonly = 1;
 	_starpu_spin_unlock(&initial_handle->header_lock);
 
@@ -719,7 +909,7 @@ void starpu_data_unpartition_readonly_submit(starpu_data_handle_t initial_handle
 	struct starpu_data_descr descr[nparts];
 	for (i = 0, n = 0; i < nparts; i++)
 	{
-		STARPU_ASSERT_MSG(children[i]->father_handle == initial_handle, "children parameter of starpu_data_partition_submit must be the children of the parent parameter");
+		STARPU_ASSERT_MSG(children[i]->father_handle == initial_handle, "child(%d) %p is partitioned from %p and not from the given parameter %p", i, children[i], children[i]->father_handle, initial_handle);
 		if (!children[i]->initialized)
 			/* Dropped value, do not care about coherency for this one */
 			continue;
@@ -731,14 +921,131 @@ void starpu_data_unpartition_readonly_submit(starpu_data_handle_t initial_handle
 	starpu_task_insert(initial_handle->switch_cl, STARPU_W, initial_handle, STARPU_DATA_MODE_ARRAY, descr, n, 0);
 }
 
-/*
- * Given an integer N, NPARTS the number of parts it must be divided in, ID the
- * part currently considered, determines the CHUNK_SIZE and the OFFSET, taking
- * into account the size of the elements stored in the data structure ELEMSIZE
- * and LD, the leading dimension.
- */
+/* Unpartition everything below ancestor */
+void starpu_data_unpartition_submit_r(starpu_data_handle_t ancestor, int gathering_node)
+{
+	unsigned i, j, nsiblings;
+	if (!ancestor->partitioned)
+		/* It's already unpartitioned */
+		return;
+	_STARPU_DEBUG("ancestor %p needs unpartitioning\n", ancestor);
+	if (ancestor->readonly)
+	{
+		unsigned n = ancestor->partitioned;
+		/* Uh, has to go through all read-only partitions */
+		for (i = 0; i < n; i++)
+		{
+			/* Note: active_readonly_children is emptied by starpu_data_unpartition_submit calls */
+			starpu_data_handle_t *children = ancestor->active_readonly_children[0];
+			_STARPU_DEBUG("unpartition readonly children %p etc.\n", children[0]);
+			nsiblings = children[0]->nsiblings;
+			for (j = 0; j < nsiblings; j++)
+			{
+				/* Make sure our children are unpartitioned */
+				starpu_data_unpartition_submit_r(children[j], gathering_node);
+			}
+			/* And unpartition them */
+			starpu_data_unpartition_submit(ancestor, nsiblings, children, gathering_node);
+		}
+	}
+	else
+	{
+		_STARPU_DEBUG("unpartition children %p\n", ancestor->active_children);
+		/* Only one partition */
+		nsiblings = ancestor->active_children[0]->nsiblings;
+		for (i = 0; i < nsiblings; i++)
+			starpu_data_unpartition_submit_r(ancestor->active_children[i], gathering_node);
+		/* And unpartition ourself */
+		starpu_data_unpartition_submit(ancestor, nsiblings, ancestor->active_children, gathering_node);
+	}
+}
+
+/* Make ancestor partition itself properly for target */
+static void _starpu_data_partition_access_look_up(starpu_data_handle_t ancestor, starpu_data_handle_t target, int write)
+{
+	/* First make sure ancestor has proper state, if not, ask father */
+	if (!ancestor->active || (write && ancestor->active_ro))
+	{
+		/* (The root is always active-rw) */
+		STARPU_ASSERT(ancestor->father_handle);
+		_STARPU_DEBUG("ancestor %p is not ready: %s, asking father %p\n", ancestor, ancestor->active ? ancestor->active_ro ? "RO" : "RW" : "NONE", ancestor->father_handle);
+		_starpu_data_partition_access_look_up(ancestor->father_handle, ancestor, write);
+		_STARPU_DEBUG("ancestor %p is now ready\n", ancestor);
+	}
+	else
+		_STARPU_DEBUG("ancestor %p was ready\n", ancestor);
+
+	/* We shouldn't be called for nothing */
+	STARPU_ASSERT(!ancestor->partitioned || !target || ancestor->active_children != target->siblings || (ancestor->readonly && write));
+
+	/* Then unpartition ancestor if needed */
+	if (ancestor->partitioned &&
+			/* Not the right children, unpartition ourself */
+			((target && write && ancestor->active_children != target->siblings) ||
+			 (target && !write && !ancestor->readonly) ||
+			/* We are partitioned and we want to write or some child
+			 * is writing and we want to read, unpartition ourself*/
+			(!target && (write || !ancestor->readonly))))
+	{
+#ifdef STARPU_DEVEL
+#warning FIXME: better choose gathering node
+#endif
+		starpu_data_unpartition_submit_r(ancestor, STARPU_MAIN_RAM);
+	}
+
+	if (!target)
+	{
+		_STARPU_DEBUG("ancestor %p is done\n", ancestor);
+		/* No child target, nothing more to do actually.  */
+		return;
+	}
+
+	/* Then partition ancestor towards target, if needed */
+	if (ancestor->partitioned)
+	{
+		/* That must be readonly, otherwise we would have unpartitioned it */
+		STARPU_ASSERT(ancestor->readonly);
+		if (write)
+		{
+			_STARPU_DEBUG("ancestor %p is already partitioned RO, turn RW\n", ancestor);
+			/* Already partitioned, normally it's already for the target */
+			STARPU_ASSERT(ancestor->active_children == target->siblings);
+			/* And we are here just because we haven't partitioned rw */
+			STARPU_ASSERT(ancestor->readonly && write);
+			/* So we just need to upgrade ro to rw */
+			starpu_data_partition_readwrite_upgrade_submit(ancestor, target->nsiblings, target->siblings);
+		}
+		else
+		{
+			_STARPU_DEBUG("ancestor %p is already partitioned RO, but not to target, partition towards target too\n", ancestor);
+			/* So we just need to upgrade ro to rw */
+			starpu_data_partition_readonly_submit(ancestor, target->nsiblings, target->siblings);
+		}
+	}
+	else
+	{
+		/* Just need to partition properly for the child */
+		if (write)
+		{
+			_STARPU_DEBUG("partition ancestor %p RW\n", ancestor);
+			starpu_data_partition_submit(ancestor, target->nsiblings, target->siblings);
+		}
+		else
+		{
+			_STARPU_DEBUG("partition ancestor %p RO\n", ancestor);
+			starpu_data_partition_readonly_submit(ancestor, target->nsiblings, target->siblings);
+		}
+	}
+}
+
+void _starpu_data_partition_access_submit(starpu_data_handle_t target, int write)
+{
+	_STARPU_DEBUG("accessing %p %s\n", target, write ? "RW" : "RO");
+	_starpu_data_partition_access_look_up(target, NULL, write);
+}
+
 void
-_starpu_filter_nparts_compute_chunk_size_and_offset(unsigned n, unsigned nparts,
+starpu_filter_nparts_compute_chunk_size_and_offset(unsigned n, unsigned nparts,
 					     size_t elemsize, unsigned id,
 					     unsigned ld, unsigned *chunk_size,
 					     size_t *offset)
@@ -763,3 +1070,8 @@ _starpu_filter_nparts_compute_chunk_size_and_offset(unsigned n, unsigned nparts,
 	if (offset != NULL)
 		*offset = (id *(n/nparts) + STARPU_MIN(remainder, id)) * ld * elemsize;
 }
+
+void starpu_data_partition_not_automatic(starpu_data_handle_t handle)
+{
+	handle->partition_automatic_disabled = 1;
+}
