@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2019                                Université de Bordeaux
- * Copyright (C) 2011,2012,2014,2017                      Inria
- * Copyright (C) 2010-2015,2017,2019                      CNRS
+ * Copyright (C) 2008-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -138,8 +136,11 @@ void starpu_vector_data_register_allocsize(starpu_data_handle_t *handleptr, int
 #if (!defined(STARPU_SIMGRID) && !defined(STARPU_OPENMP))
 	if (home_node >= 0 && starpu_node_get_kind(home_node) == STARPU_CPU_RAM)
 	{
-		STARPU_ASSERT_ACCESSIBLE(ptr);
-		STARPU_ASSERT_ACCESSIBLE(ptr + nx*elemsize - 1);
+		if (nx && elemsize)
+		{
+			STARPU_ASSERT_ACCESSIBLE(ptr);
+			STARPU_ASSERT_ACCESSIBLE(ptr + nx*elemsize - 1);
+		}
 	}
 #endif
 
@@ -188,7 +189,7 @@ static int vector_alloc_compare(void *data_interface_a, void *data_interface_b)
 	struct starpu_vector_interface *vector_a = (struct starpu_vector_interface *) data_interface_a;
 	struct starpu_vector_interface *vector_b = (struct starpu_vector_interface *) data_interface_b;
 
-	/* Two vectors are considered compatible if they have the same size */
+	/* Two vectors are considered allocation-compatible if they have the same size */
 	return (vector_a->allocsize == vector_b->allocsize);
 }
 
