@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010  Université de Bordeaux 1
- * Copyright (C) 2010  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -20,7 +19,7 @@
 #include <common/config.h>
 #include <datawizard/coherency.h>
 
-void _starpu_benchmark_ping_pong(starpu_data_handle handle,
+void _starpu_benchmark_ping_pong(starpu_data_handle_t handle,
 			unsigned node0, unsigned node1, unsigned niter)
 {
 	/* We assume that no one is using that handle !! */
@@ -29,12 +28,24 @@ void _starpu_benchmark_ping_pong(starpu_data_handle handle,
 	{
 		int ret;
 
-		struct starpu_data_replicate_s *replicate_0 = &handle->per_node[node0];
-		ret = _starpu_fetch_data_on_node(handle, replicate_0, STARPU_RW, 0, NULL, NULL);
-		STARPU_ASSERT(!ret);
+		_starpu_spin_lock(&handle->header_lock);
+		handle->refcnt++;
+		handle->busy_count++;
+		_starpu_spin_unlock(&handle->header_lock);
 
-		struct starpu_data_replicate_s *replicate_1 = &handle->per_node[node1];
-		ret = _starpu_fetch_data_on_node(handle, replicate_1, STARPU_RW, 0, NULL, NULL);
+		struct _starpu_data_replicate *replicate_0 = &handle->per_node[node0];
+		ret = _starpu_fetch_data_on_node(handle, node0, replicate_0, STARPU_RW, 0, STARPU_FETCH, 0, NULL, NULL, 0, "_starpu_benchmark_ping_pong");
 		STARPU_ASSERT(!ret);
+		_starpu_release_data_on_node(handle, 0, replicate_0);
+
+		_starpu_spin_lock(&handle->header_lock);
+		handle->refcnt++;
+		handle->busy_count++;
+		_starpu_spin_unlock(&handle->header_lock);
+
+		struct _starpu_data_replicate *replicate_1 = &handle->per_node[node1];
+		ret = _starpu_fetch_data_on_node(handle, node1, replicate_1, STARPU_RW, 0, STARPU_FETCH, 0, NULL, NULL, 0, "_starpu_benchmark_ping_pong");
+		STARPU_ASSERT(!ret);
+		_starpu_release_data_on_node(handle, 0, replicate_1);
 	}
 }
