@@ -1,8 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2016-2017                                CNRS
- * Copyright (C) 2015                                     Université de Bordeaux
- * Copyright (C) 2015                                     Anthony Simonet
+ * Copyright (C) 2015-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2015       Anthony Simonet
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -23,7 +22,8 @@
 #include "starpu_fxt.h"
 
 #ifdef STARPU_USE_FXT
-static struct component {
+static struct component
+{
 	UT_hash_handle hh;
 	char *name;
 	int workerid;
@@ -45,7 +45,7 @@ static unsigned nflowing;
 
 static struct component *fxt_component_root(void)
 {
-	struct component *comp, *tmp;
+	struct component *comp=NULL, *tmp=NULL;
 	HASH_ITER(hh, components, comp, tmp)
 	{
 		while (comp->parent)
@@ -408,10 +408,13 @@ void _starpu_fxt_component_push(FILE *output, struct starpu_fxt_options *options
 	}
 	else
 		nflowing++;
-	if (prio)
-		to_p->npriotasks++;
-	else
-		to_p->ntasks++;
+	if (to_p)
+	{
+		if (prio)
+			to_p->npriotasks++;
+		else
+			to_p->ntasks++;
+	}
 
 	// fprintf(stderr,"push from %s to %s\n", from_p?from_p->name:"none", to_p?to_p->name:"none");
 	fxt_component_print_step(output, options, timestamp, workerid, 1, from_p, to_p);
@@ -434,10 +437,13 @@ void _starpu_fxt_component_pull(FILE *output, struct starpu_fxt_options *options
 		COMPONENT_FIND(components, to, to_p);
 		STARPU_ASSERT(to_p);
 	}
-	if (prio)
-		from_p->npriotasks--;
-	else
-		from_p->ntasks--;
+	if (from_p)
+	{
+		if (prio)
+			from_p->npriotasks--;
+		else
+			from_p->ntasks--;
+	}
 	if (to_p)
 	{
 		if (prio)
