@@ -1,11 +1,10 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011,2012,2014-2017                      Inria
- * Copyright (C) 2008-2019                                Université de Bordeaux
- * Copyright (C) 2010                                     Mehdi Juhoor
- * Copyright (C) 2010-2017,2019                           CNRS
- * Copyright (C) 2013                                     Thibaut Lambert
- * Copyright (C) 2011                                     Télécom-SudParis
+ * Copyright (C) 2008-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010       Mehdi Juhoor
+ * Copyright (C) 2011       Télécom-SudParis
+ * Copyright (C) 2013       Thibaut Lambert
+ * Copyright (C) 2020       Federal University of Rio Grande do Sul (UFRGS)
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -139,7 +138,7 @@ static int execute_job_on_cpu(struct _starpu_job *j, struct starpu_task *worker_
 		{
 			/* Wait for other threads to exit barrier_wait so we
 			 * can safely drop the job structure */
-			MSG_process_sleep(0.0000001);
+			starpu_sleep(0.0000001);
 			j->after_work_busy_barrier = 0;
 		}
 #else
@@ -295,6 +294,7 @@ int _starpu_cpu_driver_run_once(struct _starpu_worker *cpu_worker)
 	if (pending_task != NULL && cpu_worker->nb_buffers_transferred == cpu_worker->nb_buffers_totransfer)
 	{
 		int ret;
+		STARPU_RMB();
 		_STARPU_TRACE_END_PROGRESS(memnode);
 		j = _starpu_get_job_associated_to_task(pending_task);
 
@@ -331,7 +331,7 @@ int _starpu_cpu_driver_run_once(struct _starpu_worker *cpu_worker)
 	}
   #else
 	/* Previous simgrid versions don't really permit to use wait_timedwait in C */
-	MSG_process_sleep(0.001);
+	starpu_sleep(0.001);
   #endif
  #endif
 #endif
@@ -444,7 +444,7 @@ struct _starpu_driver_ops _starpu_driver_cpu_ops =
 };
 #endif /* STARPU_USE_CPU */
 
-int _starpu_cpu_copy_data(starpu_data_handle_t handle, void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, struct _starpu_data_request *req)
+int _starpu_cpu_copy_interface(starpu_data_handle_t handle, void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, struct _starpu_data_request *req)
 {
 	int src_kind = starpu_node_get_kind(src_node);
 	int dst_kind = starpu_node_get_kind(dst_node);
@@ -459,7 +459,7 @@ int _starpu_cpu_copy_data(starpu_data_handle_t handle, void *src_interface, unsi
 	return ret;
 }
 
-int _starpu_cpu_copy_interface(uintptr_t src, size_t src_offset, unsigned src_node, uintptr_t dst, size_t dst_offset, unsigned dst_node, size_t size, struct _starpu_async_channel *async_channel)
+int _starpu_cpu_copy_data(uintptr_t src, size_t src_offset, unsigned src_node, uintptr_t dst, size_t dst_offset, unsigned dst_node, size_t size, struct _starpu_async_channel *async_channel)
 {
 	int src_kind = starpu_node_get_kind(src_node);
 	int dst_kind = starpu_node_get_kind(dst_node);
@@ -509,30 +509,6 @@ void _starpu_cpu_free_on_node(unsigned dst_node, uintptr_t addr, size_t size, in
 
 struct _starpu_node_ops _starpu_driver_cpu_node_ops =
 {
-	.copy_data_to[STARPU_UNUSED] = NULL,
-	.copy_data_to[STARPU_CPU_RAM] = _starpu_cpu_copy_data,
-#ifdef STARPU_USE_CUDA
-	.copy_data_to[STARPU_CUDA_RAM] = _starpu_cuda_copy_data_from_cpu_to_cuda,
-#else
-	.copy_data_to[STARPU_CUDA_RAM] = NULL,
-#endif
-#ifdef STARPU_USE_OPENCL
-	.copy_data_to[STARPU_OPENCL_RAM] = _starpu_opencl_copy_data_from_cpu_to_opencl,
-#else
-	.copy_data_to[STARPU_OPENCL_RAM] = NULL,
-#endif
-	.copy_data_to[STARPU_DISK_RAM] = _starpu_disk_copy_data_from_cpu_to_disk,
-#ifdef STARPU_USE_MIC
-	.copy_data_to[STARPU_MIC_RAM] = _starpu_mic_copy_data_from_cpu_to_mic,
-#else
-	.copy_data_to[STARPU_MIC_RAM] = NULL,
-#endif
-#ifdef STARPU_USE_MPI_MASTER_SLAVE
-	.copy_data_to[STARPU_MPI_MS_RAM] = _starpu_mpi_copy_data_from_cpu_to_mpi,
-#else
-	.copy_data_to[STARPU_MPI_MS_RAM] = NULL,
-#endif
-
 	.copy_interface_to[STARPU_UNUSED] = NULL,
 	.copy_interface_to[STARPU_CPU_RAM] = _starpu_cpu_copy_interface,
 #ifdef STARPU_USE_CUDA
@@ -557,6 +533,58 @@ struct _starpu_node_ops _starpu_driver_cpu_node_ops =
 	.copy_interface_to[STARPU_MPI_MS_RAM] = NULL,
 #endif
 
+	.copy_data_to[STARPU_UNUSED] = NULL,
+	.copy_data_to[STARPU_CPU_RAM] = _starpu_cpu_copy_data,
+#ifdef STARPU_USE_CUDA
+	.copy_data_to[STARPU_CUDA_RAM] = _starpu_cuda_copy_data_from_cpu_to_cuda,
+#else
+	.copy_data_to[STARPU_CUDA_RAM] = NULL,
+#endif
+#ifdef STARPU_USE_OPENCL
+	.copy_data_to[STARPU_OPENCL_RAM] = _starpu_opencl_copy_data_from_cpu_to_opencl,
+#else
+	.copy_data_to[STARPU_OPENCL_RAM] = NULL,
+#endif
+	.copy_data_to[STARPU_DISK_RAM] = _starpu_disk_copy_data_from_cpu_to_disk,
+#ifdef STARPU_USE_MIC
+	.copy_data_to[STARPU_MIC_RAM] = _starpu_mic_copy_data_from_cpu_to_mic,
+#else
+	.copy_data_to[STARPU_MIC_RAM] = NULL,
+#endif
+#ifdef STARPU_USE_MPI_MASTER_SLAVE
+	.copy_data_to[STARPU_MPI_MS_RAM] = _starpu_mpi_copy_data_from_cpu_to_mpi,
+#else
+	.copy_data_to[STARPU_MPI_MS_RAM] = NULL,
+#endif
+
+	.copy2d_data_to[STARPU_UNUSED] = NULL,
+	.copy2d_data_to[STARPU_CPU_RAM] = NULL,
+#ifdef STARPU_USE_CUDA
+	.copy2d_data_to[STARPU_CUDA_RAM] = _starpu_cuda_copy2d_data_from_cpu_to_cuda,
+#else
+	.copy2d_data_to[STARPU_CUDA_RAM] = NULL,
+#endif
+	.copy2d_data_to[STARPU_OPENCL_RAM] = NULL,
+	.copy2d_data_to[STARPU_DISK_RAM] = NULL,
+	.copy2d_data_to[STARPU_MIC_RAM] = NULL,
+	.copy2d_data_to[STARPU_MPI_MS_RAM] = NULL,
+
+	.copy3d_data_to[STARPU_UNUSED] = NULL,
+	.copy3d_data_to[STARPU_CPU_RAM] = NULL,
+#if 0
+#ifdef STARPU_USE_CUDA
+	.copy3d_data_to[STARPU_CUDA_RAM] = _starpu_cuda_copy3d_data_from_cpu_to_cuda,
+#else
+	.copy3d_data_to[STARPU_CUDA_RAM] = NULL,
+#endif
+#else
+	.copy3d_data_to[STARPU_CUDA_RAM] = NULL,
+#endif
+	.copy3d_data_to[STARPU_OPENCL_RAM] = NULL,
+	.copy3d_data_to[STARPU_DISK_RAM] = NULL,
+	.copy3d_data_to[STARPU_MIC_RAM] = NULL,
+	.copy3d_data_to[STARPU_MPI_MS_RAM] = NULL,
+
 	.wait_request_completion = NULL,
 	.test_request_completion = NULL,
 	.is_direct_access_supported = _starpu_cpu_is_direct_access_supported,
