@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2011,2013-2017                      Université de Bordeaux
- * Copyright (C) 2010-2013,2015-2017                      CNRS
+ * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -18,18 +17,26 @@
 #include <starpu.h>
 #include <starpu_profiling.h>
 #include <profiling/profiling.h>
+#include <datawizard/memory_nodes.h>
+
+static double convert_to_GB(float d)
+{
+	const double divisor = 1024;
+	return d = (((d / divisor) / divisor) / divisor);
+}
 
 void _starpu_profiling_bus_helper_display_summary(FILE *stream)
 {
 	int long long sum_transferred = 0;
 
-	fprintf(stream, "\nData transfer statistics:\n");
-	fprintf(stream,   "*************************\n");
+	fprintf(stream, "\n#---------------------\n");
+	fprintf(stream, "Data transfer stats:\n");
 
 	int busid;
 	int bus_cnt = starpu_bus_get_count();
 	for (busid = 0; busid < bus_cnt; busid++)
 	{
+		char src_name[128], dst_name[128];
 		int src, dst;
 
 		src = starpu_bus_get_src(busid);
@@ -40,21 +47,42 @@ void _starpu_profiling_bus_helper_display_summary(FILE *stream)
 
 		int long long transferred = bus_info.transferred_bytes;
 		int long long transfer_cnt =  bus_info.transfer_count;
-		double elapsed_time = starpu_timing_timespec_to_us(&bus_info.total_time);
+		double elapsed_time = starpu_timing_timespec_to_us(&bus_info.total_time) / 1e6;
 
-		fprintf(stream, "\t%d -> %d\t%.2lf MB\t%.2lfMB/s\t(transfers : %lld - avg %.2lf MB)\n", src, dst, (1.0*transferred)/(1024*1024), transferred/elapsed_time, transfer_cnt, (1.0*transferred)/(transfer_cnt*1024*1024));
+		double d = convert_to_GB(transferred);
+
+		starpu_memory_node_get_name(src, src_name, sizeof(src_name));
+		starpu_memory_node_get_name(dst, dst_name, sizeof(dst_name));
+
+		fprintf(stream, "\t%s -> %s", src_name, dst_name);
+		fprintf(stream, "\t%.4lf %s", d, "GB");
+		fprintf(stream, "\t%.4lf %s/s", (d * 1024) / elapsed_time, "MB");
+		fprintf(stream, "\t(transfers : %lld - avg %.4lf %s)\n", transfer_cnt, (d * 1024) / transfer_cnt, "MB");
 
 		sum_transferred += transferred;
 	}
 
-	fprintf(stream, "Total transfers: %.2lf MB\n", (1.0*sum_transferred)/(1024*1024));
+	double d = convert_to_GB(sum_transferred);
+
+	fprintf(stream, "Total transfers: %.4lf %s\n", d, "GB");
+	fprintf(stream, "#---------------------\n");
 }
 
 void starpu_profiling_bus_helper_display_summary(void)
 {
 	const char *stats;
 	if (!((stats = starpu_getenv("STARPU_BUS_STATS")) && atoi(stats))) return;
-	_starpu_profiling_bus_helper_display_summary(stderr);
+	const char *filename = starpu_getenv("STARPU_BUS_STATS_FILE");
+	if (filename==NULL)
+		_starpu_profiling_bus_helper_display_summary(stderr);
+	else
+	{
+		FILE *sfile = fopen(filename, "w+");
+		STARPU_ASSERT_MSG(sfile, "Could not open file %s for displaying bus stats (%s). You can specify another file destination with the STARPU_BUS_STATS_FILE environment variable", filename, strerror(errno));
+		_starpu_profiling_bus_helper_display_summary(sfile);
+		fclose(sfile);
+	}
+
 }
 
 void _starpu_profiling_worker_helper_display_summary(FILE *stream)
@@ -65,8 +93,8 @@ void _starpu_profiling_worker_helper_display_summary(FILE *stream)
 	int workerid;
 	int worker_cnt = starpu_worker_get_count();
 
-	fprintf(stream, "\nWorker statistics:\n");
-	fprintf(stream,   "******************\n");
+	fprintf(stream, "\n#---------------------\n");
+	fprintf(stream, "Worker stats:\n");
 
 	for (workerid = 0; workerid < worker_cnt; workerid++)
 	{
@@ -76,6 +104,9 @@ void _starpu_profiling_worker_helper_display_summary(FILE *stream)
 
 		starpu_worker_get_name(workerid, name, sizeof(name));
 
+		fprintf(stream, "%-32s\n", name);
+		fprintf(stream, "\t%d task(s)\n", info.executed_tasks);
+
 		if (profiling)
 		{
 			double total_time = starpu_timing_timespec_to_us(&info.total_time) / 1000.;
@@ -84,16 +115,14 @@ void _starpu_profiling_worker_helper_display_summary(FILE *stream)
 			if (total_time > overall_time)
 				overall_time = total_time;
 
-			fprintf(stream, "%-32s\n", name);
-			fprintf(stream, "\t%d task(s)\n\ttotal: %.2lf ms executing: %.2lf ms sleeping: %.2lf ms overhead %.2lf ms\n", info.executed_tasks, total_time, executing_time, sleeping_time, total_time - executing_time - sleeping_time);
+			fprintf(stream, "\ttotal: %.2lf ms executing: %.2lf ms sleeping: %.2lf ms overhead %.2lf ms\n",
+				total_time, executing_time, sleeping_time, total_time - executing_time - sleeping_time);
 			if (info.used_cycles || info.stall_cycles)
 				fprintf(stream, "\t%llu Mcy %llu Mcy stall\n", (unsigned long long)info.used_cycles/1000000, (unsigned long long)info.stall_cycles/1000000);
 			if (info.energy_consumed)
 				fprintf(stream, "\t%f J consumed\n", info.energy_consumed);
-		}
-		else
-		{
-			fprintf(stream, "\t%-32s\t%d task(s)\n", name, info.executed_tasks);
+			if (info.flops)
+				fprintf(stream, "\t%f GFlop/s\n\n", info.flops / total_time / 1000000);
 		}
 
 		sum_consumed += info.energy_consumed;
@@ -108,16 +137,25 @@ void _starpu_profiling_worker_helper_display_summary(FILE *stream)
 			double idle_energy = idle_power * overall_time / 1000.; /* J */
 
 			fprintf(stream, "Idle energy: %.2lf J\n", idle_energy);
-			sum_consumed += idle_energy;
+			fprintf(stream, "Total energy: %.2lf J\n",
+				sum_consumed + idle_energy);
 		}
 	}
-	if (profiling && sum_consumed)
-		fprintf(stream, "Total consumption: %.2lf J\n", sum_consumed);
+	fprintf(stream, "#---------------------\n");
 }
 
 void starpu_profiling_worker_helper_display_summary(void)
 {
 	const char *stats;
 	if (!((stats = starpu_getenv("STARPU_WORKER_STATS")) && atoi(stats))) return;
-	_starpu_profiling_worker_helper_display_summary(stderr);
+	const char *filename = starpu_getenv("STARPU_WORKER_STATS_FILE");
+	if (filename==NULL)
+		_starpu_profiling_worker_helper_display_summary(stderr);
+	else
+	{
+		FILE *sfile = fopen(filename, "w+");
+		STARPU_ASSERT_MSG(sfile, "Could not open file %s for displaying worker stats (%s). You can specify another file destination with the STARPU_WORKER_STATS_FILE environment variable", filename, strerror(errno));
+		_starpu_profiling_worker_helper_display_summary(sfile);
+		fclose(sfile);
+	}
 }
