@@ -1,9 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2013                                     Inria
- * Copyright (C) 2014-2015,2017                           CNRS
- * Copyright (C) 2014-2016                                Université de Bordeaux
- * Copyright (C) 2013                                     Simon Archipoff
+ * Copyright (C) 2013-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013       Simon Archipoff
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -17,10 +15,11 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
+#include <float.h>
+
 #include <starpu_sched_component.h>
 #include <starpu_scheduler.h>
-
-#include <float.h>
+#include <core/workers.h>
 
 /* return true if workerid can execute task, and fill task->predicted and task->predicted_transfer
  *  according to best implementation predictions
@@ -29,32 +28,41 @@ static int find_best_impl(unsigned sched_ctx_id, struct starpu_task * task, int
 {
 	double len = DBL_MAX;
 	int best_impl = -1;
-	int impl;
-	for(impl = 0; impl < STARPU_MAXIMPLEMENTATIONS; impl++)
+	unsigned impl;
+	if (!task->cl->model)
 	{
-		if(starpu_worker_can_execute_task(workerid, task, impl))
+		/* No perfmodel, first available will be fine */
+		starpu_worker_can_execute_task_first_impl(workerid, task, &impl);
+		best_impl = impl;
+		len = 0.0;
+	}
+	else
+	{
+		struct starpu_perfmodel_arch* archtype = starpu_worker_get_perf_archtype(workerid, sched_ctx_id);
+		for(impl = 0; impl < STARPU_MAXIMPLEMENTATIONS; impl++)
 		{
-			struct starpu_perfmodel_arch* archtype = starpu_worker_get_perf_archtype(workerid, sched_ctx_id);
-			double d = starpu_task_expected_length(task, archtype, impl);
-			if(isnan(d))
+			if(starpu_worker_can_execute_task(workerid, task, impl))
 			{
-				best_impl = impl;
-				len = 0.0;
-				break;
-			}
-			if(d < len)
-			{
-				len = d;
-				best_impl = impl;
+				double d = starpu_task_expected_length(task, archtype, impl);
+				if(isnan(d))
+				{
+					best_impl = impl;
+					len = 0.0;
+					break;
+				}
+				if(d < len)
+				{
+					len = d;
+					best_impl = impl;
+				}
 			}
 		}
 	}
 	if(best_impl == -1)
 		return 0;
 
-	int memory_node = starpu_worker_get_memory_node(workerid);
 	task->predicted = len;
-	task->predicted_transfer = starpu_task_expected_data_transfer_time(memory_node, task);
+	task->predicted_transfer = starpu_task_expected_data_transfer_time_for(task, workerid);
 	starpu_task_set_implementation(task, best_impl);
 	return 1;
 }
@@ -85,10 +93,10 @@ int starpu_sched_component_is_best_implementation(struct starpu_sched_component
 	return component->push_task == best_implementation_push_task;
 }
 
-static struct starpu_task * best_implementation_pull_task(struct starpu_sched_component * component)
+static struct starpu_task * best_implementation_pull_task(struct starpu_sched_component * component, struct starpu_sched_component * from STARPU_ATTRIBUTE_UNUSED)
 {
 	struct starpu_task * task = NULL;
-	int i;
+	unsigned i;
 	for(i=0; i < component->nparents; i++)
 	{
 		if(component->parents[i] == NULL)
@@ -106,8 +114,9 @@ static struct starpu_task * best_implementation_pull_task(struct starpu_sched_co
 	return task;
 }
 
-struct starpu_sched_component * starpu_sched_component_best_implementation_create(struct starpu_sched_tree *tree, void * arg STARPU_ATTRIBUTE_UNUSED)
+struct starpu_sched_component * starpu_sched_component_best_implementation_create(struct starpu_sched_tree *tree, void *arg)
 {
+	(void)arg;
 	struct starpu_sched_component * component = starpu_sched_component_create(tree, "best_impl");
 	component->push_task = best_implementation_push_task;
 	component->pull_task = best_implementation_pull_task;
