@@ -1,9 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2013-2014                                Inria
- * Copyright (C) 2014-2015,2017                           CNRS
- * Copyright (C) 2014-2018                                Université de Bordeaux
- * Copyright (C) 2013                                     Simon Archipoff
+ * Copyright (C) 2013-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013       Simon Archipoff
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -44,7 +42,7 @@ int starpu_sched_component_execute_preds(struct starpu_sched_component * compone
 	int can_execute = 0;
 	starpu_task_bundle_t bundle = task->bundle;
 	double len = DBL_MAX;
-	
+
 
 	int workerid;
 	for(workerid = starpu_bitmap_first(component->workers_in_ctx);
@@ -68,12 +66,12 @@ int starpu_sched_component_execute_preds(struct starpu_sched_component * compone
 				{
 					*length = d;
 					return can_execute;
-						
 				}
 				if(_STARPU_IS_ZERO(d))
 				{
 					continue;
 				}
+				STARPU_ASSERT_MSG(d >= 0, "workerid=%d, nimpl=%d, bundle=%p, d=%lf\n", workerid, nimpl, bundle, d);
 				if(d < len)
 				{
 					len = d;
@@ -85,7 +83,7 @@ int starpu_sched_component_execute_preds(struct starpu_sched_component * compone
 	}
 
 	if(len == DBL_MAX) /* we dont have perf model */
-		len = 0.0; 
+		len = 0.0;
 	if(length)
 		*length = len;
 	return can_execute;
@@ -147,13 +145,13 @@ double starpu_sched_component_transfer_length(struct starpu_sched_component * co
 }
 
 /* This function can be called by components when they think that a prefetching request can be submitted.
- * For example, it is currently used by the MCT component to begin the prefetching on accelerators 
+ * For example, it is currently used by the MCT component to begin the prefetching on accelerators
  * on which it pushed tasks as soon as possible.
  */
 void starpu_sched_component_prefetch_on_node(struct starpu_sched_component * component, struct starpu_task * task)
 {
 	if (starpu_get_prefetch_flag() && (!task->prefetched)
-		&& (component->properties >= STARPU_SCHED_COMPONENT_SINGLE_MEMORY_NODE))
+		&& (component->properties & STARPU_SCHED_COMPONENT_SINGLE_MEMORY_NODE))
 	{
 		int worker = starpu_bitmap_first(component->workers_in_ctx);
 		unsigned memory_node = starpu_worker_get_memory_node(worker);
@@ -163,13 +161,13 @@ void starpu_sched_component_prefetch_on_node(struct starpu_sched_component * com
 }
 
 /* remove all child
- * for all child of component, if child->parents[x] == component, set child->parents[x] to null 
+ * for all child of component, if child->parents[x] == component, set child->parents[x] to null
  * call component->deinit_data
  */
 void starpu_sched_component_destroy(struct starpu_sched_component *component)
 {
 	STARPU_ASSERT(component);
-	int i,j;
+	unsigned i,j;
 	for(i = 0; i < component->nchildren; i++)
 	{
 		struct starpu_sched_component * child = component->children[i];
@@ -204,7 +202,7 @@ void starpu_sched_component_destroy_rec(struct starpu_sched_component * componen
 	if(component == NULL)
 		return;
 
-	int i = 0;
+	unsigned i = 0;
 	while(i < component->nchildren)
 	{
 		if (starpu_sched_component_is_worker(component->children[i]))
@@ -225,6 +223,8 @@ void set_properties(struct starpu_sched_component * component)
 	int worker = starpu_bitmap_first(component->workers_in_ctx);
 	if (worker == -1)
 		return;
+	if (starpu_worker_is_combined_worker(worker))
+		return;
 #ifdef STARPU_DEVEL
 #warning FIXME: Not all CUDA devices have the same speed
 #endif
@@ -234,14 +234,16 @@ void set_properties(struct starpu_sched_component * component)
 	int is_all_same_component = 1;
 	for(;
 	    worker != -1;
-	    worker = starpu_bitmap_next(component->workers_in_ctx, worker))		
+	    worker = starpu_bitmap_next(component->workers_in_ctx, worker))
 	{
+		if(starpu_worker_is_combined_worker(worker))
+			continue;
 		if(first_worker != _starpu_get_worker_struct(worker)->worker_mask)
 			is_homogeneous = 0;
 		if(first_memory_node != _starpu_get_worker_struct(worker)->memory_node)
 			is_all_same_component = 0;
 	}
-	
+
 
 	if(is_homogeneous)
 		component->properties |= STARPU_SCHED_COMPONENT_HOMOGENEOUS;
@@ -258,13 +260,13 @@ void _starpu_sched_component_update_workers(struct starpu_sched_component * comp
 	if(starpu_sched_component_is_worker(component))
 		return;
 	starpu_bitmap_unset_all(component->workers);
-	int i;
+	unsigned i;
 	for(i = 0; i < component->nchildren; i++)
 	{
 		_starpu_sched_component_update_workers(component->children[i]);
 		starpu_bitmap_or(component->workers, component->children[i]->workers);
-		component->notify_change_workers(component);
 	}
+	component->notify_change_workers(component);
 }
 
 /* recursively set the component->workers_in_ctx in component's subtree
@@ -272,21 +274,35 @@ void _starpu_sched_component_update_workers(struct starpu_sched_component * comp
 void _starpu_sched_component_update_workers_in_ctx(struct starpu_sched_component * component, unsigned sched_ctx_id)
 {
 	STARPU_ASSERT(component);
+	/* worker components are shared among sched_ctxs, thus we do not apply the sched_ctx worker mask to them.
+	 * per-ctx filtering is performed higher in the tree */
 	if(starpu_sched_component_is_worker(component))
 		return;
 	struct starpu_bitmap * workers_in_ctx = _starpu_get_worker_mask(sched_ctx_id);
 	starpu_bitmap_unset_and(component->workers_in_ctx,component->workers, workers_in_ctx);
-	int i,j;
+	unsigned i,j;
+	for(i = starpu_worker_get_count(); i < starpu_worker_get_count() + starpu_combined_worker_get_count(); i++)
+	{
+		if (starpu_bitmap_get(component->workers, i))
+		{
+			/* Component has this combined worker, check whether the
+			 * context has all the corresponding workers */
+			int worker_size;
+			int *combined_workerid;
+			starpu_combined_worker_get_description(i, &worker_size, &combined_workerid);
+			for (j = 0; j < (unsigned) worker_size; j++)
+				if (!starpu_bitmap_get(workers_in_ctx, combined_workerid[j]))
+					goto nocombined;
+			/* We have all workers, add it */
+			starpu_bitmap_set(component->workers_in_ctx, i);
+		}
+nocombined:
+		(void)0;
+	}
 	for(i = 0; i < component->nchildren; i++)
 	{
 		struct starpu_sched_component * child = component->children[i];
 		_starpu_sched_component_update_workers_in_ctx(child, sched_ctx_id);
-		for(j = 0; j < STARPU_NMAX_SCHED_CTXS; j++)
-			if(child->parents[j] == component)
-			{
-				starpu_bitmap_or(component->workers_in_ctx, child->workers_in_ctx);
-				break;
-			}
 	}
 	set_properties(component);
 	component->notify_change_workers(component);
@@ -311,13 +327,15 @@ struct starpu_bitmap * _starpu_get_worker_mask(unsigned sched_ctx_id)
 void starpu_sched_tree_update_workers_in_ctx(struct starpu_sched_tree * t)
 {
 	STARPU_ASSERT(t);
-	_starpu_sched_component_update_workers_in_ctx(t->root, t->sched_ctx_id);
+	if (t->root)
+		_starpu_sched_component_update_workers_in_ctx(t->root, t->sched_ctx_id);
 }
 
 void starpu_sched_tree_update_workers(struct starpu_sched_tree * t)
 {
 	STARPU_ASSERT(t);
-	_starpu_sched_component_update_workers(t->root);
+	if (t->root)
+		_starpu_sched_component_update_workers(t->root);
 }
 
 
@@ -341,8 +359,11 @@ int starpu_sched_tree_push_task(struct starpu_task * task)
 	struct starpu_sched_tree *tree = starpu_sched_ctx_get_policy_data(sched_ctx_id);
 
 	int ret_val = starpu_sched_component_push_task(NULL, tree->root,task);
-	
-	return ret_val;
+
+	/* Modular schedulers are not supposed to refuse tasks */
+	STARPU_ASSERT(!ret_val);
+
+	return 0;
 }
 
 int starpu_sched_component_push_task(struct starpu_sched_component *from STARPU_ATTRIBUTE_UNUSED, struct starpu_sched_component *to, struct starpu_task *task)
@@ -365,22 +386,56 @@ struct starpu_task * starpu_sched_tree_pop_task(unsigned sched_ctx)
 	return task;
 }
 
-struct starpu_task * starpu_sched_component_pull_task(struct starpu_sched_component *from, struct starpu_sched_component *to STARPU_ATTRIBUTE_UNUSED)
+struct starpu_task * starpu_sched_component_pull_task(struct starpu_sched_component *from, struct starpu_sched_component *to)
 {
-	struct starpu_task *task = from->pull_task(from);
+	struct starpu_task *task = from->pull_task(from, to);
 	if (task)
 		_STARPU_TRACE_SCHED_COMPONENT_PULL(from, to, task);
 	return task;
 }
 
+
+/* Pump mechanic to get the task flow rolling. Takes tasks from component and send them to the child.
+   To be used by components with only one child */
+struct starpu_task* starpu_sched_component_pump_to(struct starpu_sched_component *component, struct starpu_sched_component *child, int* success)
+{
+	int ret = 0;
+
+	struct starpu_task * task;
+
+	while (1)
+	{
+		task = component->pull_task(component,child);
+		if (!task)
+			break;
+		ret = starpu_sched_component_push_task(component,child,task);
+		if (ret)
+			break;
+		if(success)
+			* success = 1;
+	}
+	if(task && ret)
+		/* Return the task which couldn't actually be pushed */
+		return task;
+
+	return NULL;
+
+}
+
+struct starpu_task* starpu_sched_component_pump_downstream(struct starpu_sched_component *component, int* success)
+{
+	STARPU_ASSERT(component->nchildren == 1);
+	return starpu_sched_component_pump_to(component, component->children[0], success);
+}
+
 void starpu_sched_tree_add_workers(unsigned sched_ctx_id, int *workerids, unsigned nworkers)
 {
 	STARPU_ASSERT(sched_ctx_id < STARPU_NMAX_SCHED_CTXS);
 	STARPU_ASSERT(workerids);
 	struct starpu_sched_tree * t = starpu_sched_ctx_get_policy_data(sched_ctx_id);
 
-	STARPU_PTHREAD_MUTEX_LOCK(&t->lock);
-	_starpu_sched_component_lock_all_workers(sched_ctx_id);
+	STARPU_COMPONENT_MUTEX_LOCK(&t->lock);
+	_starpu_sched_component_lock_all_workers();
 
 	unsigned i;
 	for(i = 0; i < nworkers; i++)
@@ -388,8 +443,8 @@ void starpu_sched_tree_add_workers(unsigned sched_ctx_id, int *workerids, unsign
 
 	starpu_sched_tree_update_workers_in_ctx(t);
 
-	_starpu_sched_component_unlock_all_workers(sched_ctx_id);
-	STARPU_PTHREAD_MUTEX_UNLOCK(&t->lock);
+	_starpu_sched_component_unlock_all_workers();
+	STARPU_COMPONENT_MUTEX_UNLOCK(&t->lock);
 }
 
 void starpu_sched_tree_remove_workers(unsigned sched_ctx_id, int *workerids, unsigned nworkers)
@@ -398,8 +453,8 @@ void starpu_sched_tree_remove_workers(unsigned sched_ctx_id, int *workerids, uns
 	STARPU_ASSERT(workerids);
 	struct starpu_sched_tree * t = starpu_sched_ctx_get_policy_data(sched_ctx_id);
 
-	STARPU_PTHREAD_MUTEX_LOCK(&t->lock);
-	_starpu_sched_component_lock_all_workers(sched_ctx_id);
+	STARPU_COMPONENT_MUTEX_LOCK(&t->lock);
+	_starpu_sched_component_lock_all_workers();
 
 	unsigned i;
 	for(i = 0; i < nworkers; i++)
@@ -407,8 +462,8 @@ void starpu_sched_tree_remove_workers(unsigned sched_ctx_id, int *workerids, uns
 
 	starpu_sched_tree_update_workers_in_ctx(t);
 
-	_starpu_sched_component_unlock_all_workers(sched_ctx_id);
-	STARPU_PTHREAD_MUTEX_UNLOCK(&t->lock);
+	_starpu_sched_component_unlock_all_workers();
+	STARPU_COMPONENT_MUTEX_UNLOCK(&t->lock);
 }
 
 static struct starpu_sched_tree *trees[STARPU_NMAX_SCHED_CTXS];
@@ -418,8 +473,7 @@ struct starpu_sched_tree * starpu_sched_tree_create(unsigned sched_ctx_id)
 	STARPU_ASSERT(sched_ctx_id < STARPU_NMAX_SCHED_CTXS);
 	STARPU_ASSERT(!trees[sched_ctx_id]);
 	struct starpu_sched_tree *t;
-	_STARPU_MALLOC(t, sizeof(*t));
-	memset(t, 0, sizeof(*t));
+	_STARPU_CALLOC(t, 1, sizeof(*t));
 	t->sched_ctx_id = sched_ctx_id;
 	t->workers = starpu_bitmap_create();
 	STARPU_PTHREAD_MUTEX_INIT(&t->lock,NULL);
@@ -454,8 +508,8 @@ struct starpu_sched_tree * starpu_sched_tree_get(unsigned sched_ctx_id)
 void starpu_sched_component_add_child(struct starpu_sched_component* component, struct starpu_sched_component * child)
 {
 	STARPU_ASSERT(component && child);
-	STARPU_ASSERT(!starpu_sched_component_is_worker(component));
-	int i;
+	STARPU_ASSERT(!starpu_sched_component_is_simple_worker(component));
+	unsigned i;
 	for(i = 0; i < component->nchildren; i++)
 	{
 		STARPU_ASSERT(component->children[i] != component);
@@ -470,8 +524,8 @@ void starpu_sched_component_add_child(struct starpu_sched_component* component,
 static void starpu_sched_component_remove_child(struct starpu_sched_component * component, struct starpu_sched_component * child)
 {
 	STARPU_ASSERT(component && child);
-	STARPU_ASSERT(!starpu_sched_component_is_worker(component));
-	int pos;
+	STARPU_ASSERT(!starpu_sched_component_is_simple_worker(component));
+	unsigned pos;
 	for(pos = 0; pos < component->nchildren; pos++)
 		if(component->children[pos] == child)
 			break;
@@ -482,7 +536,7 @@ static void starpu_sched_component_remove_child(struct starpu_sched_component *
 static void starpu_sched_component_add_parent(struct starpu_sched_component* component, struct starpu_sched_component * parent)
 {
 	STARPU_ASSERT(component && parent);
-	int i;
+	unsigned i;
 	for(i = 0; i < component->nparents; i++)
 	{
 		STARPU_ASSERT(component->parents[i] != component);
@@ -497,7 +551,7 @@ static void starpu_sched_component_add_parent(struct starpu_sched_component* com
 static void starpu_sched_component_remove_parent(struct starpu_sched_component * component, struct starpu_sched_component * parent)
 {
 	STARPU_ASSERT(component && parent);
-	int pos;
+	unsigned pos;
 	for(pos = 0; pos < component->nparents; pos++)
 		if(component->parents[pos] == parent)
 			break;
@@ -508,11 +562,11 @@ static void starpu_sched_component_remove_parent(struct starpu_sched_component *
 /* default implementation for component->pull_task()
  * just perform a recursive call on parent
  */
-static struct starpu_task * starpu_sched_component_parents_pull_task(struct starpu_sched_component * component)
+struct starpu_task * starpu_sched_component_parents_pull_task(struct starpu_sched_component * component, struct starpu_sched_component * to STARPU_ATTRIBUTE_UNUSED)
 {
 	STARPU_ASSERT(component);
 	struct starpu_task * task = NULL;
-	int i;
+	unsigned i;
 	for(i=0; i < component->nparents; i++)
 	{
 		if(component->parents[i] == NULL)
@@ -531,18 +585,18 @@ static struct starpu_task * starpu_sched_component_parents_pull_task(struct star
  * A personally-made can_push in a component (like in prio components) is necessary to catch
  * this recursive call somewhere, if the user wants to exploit it.
  */
-static int starpu_sched_component_can_push(struct starpu_sched_component * component)
+int starpu_sched_component_can_push(struct starpu_sched_component * component, struct starpu_sched_component * to STARPU_ATTRIBUTE_UNUSED)
 {
 	STARPU_ASSERT(component);
 	int ret = 0;
 	if(component->nparents > 0)
 	{
-		int i;
+		unsigned i;
 		for(i=0; i < component->nparents; i++)
 		{
 			struct starpu_sched_component * parent = component->parents[i];
 			if(parent != NULL)
-				ret = parent->can_push(parent);
+				ret = parent->can_push(parent, component);
 			if(ret)
 				break;
 		}
@@ -554,19 +608,65 @@ static int starpu_sched_component_can_push(struct starpu_sched_component * compo
  * component. It is currenly called by components which holds a queue (like fifo and prio
  * components) to signify its childs that a task has been pushed on its local queue.
  */
-static void starpu_sched_component_can_pull(struct starpu_sched_component * component)
+int starpu_sched_component_can_pull(struct starpu_sched_component * component)
 {
 	STARPU_ASSERT(component);
 	STARPU_ASSERT(!starpu_sched_component_is_worker(component));
-	int i;
+	unsigned i;
+	for(i = 0; i < component->nchildren; i++)
+	{
+		if (component->children[i]->can_pull(component->children[i]))
+			return 1;
+	}
+	return 0;
+}
+
+
+/* A can_pull call will try to wake up one worker associated to the childs of the
+ * component. It is currenly called by components which holds a queue (like fifo and prio
+ * components) to signify its childs that a task has been pushed on its local queue.
+ */
+int starpu_sched_component_can_pull_all(struct starpu_sched_component * component)
+{
+	STARPU_ASSERT(component);
+	STARPU_ASSERT(!starpu_sched_component_is_worker(component));
+	unsigned i;
 	for(i = 0; i < component->nchildren; i++)
 		component->children[i]->can_pull(component->children[i]);
+	return 0;
 }
 
+
+/* Alternative can_pull which says that this component does not want
+   to pull but prefers that you push. It can be used by decision
+   components, in which decisions are usually taken in their push()
+   functions */
+int starpu_sched_component_send_can_push_to_parents(struct starpu_sched_component * component)
+{
+	STARPU_ASSERT(component);
+	STARPU_ASSERT(!starpu_sched_component_is_worker(component));
+
+	unsigned i;
+	int ret = 0;
+	for(i=0; i < component->nparents; i++)
+	{
+		if(component->parents[i] == NULL)
+			continue;
+		else
+		{
+			ret = component->parents[i]->can_push(component->parents[i], component);
+			if(ret)
+				break;
+		}
+	}
+	return ret != 0;
+}
+
+
 double starpu_sched_component_estimated_load(struct starpu_sched_component * component)
 {
 	double sum = 0.0;
-	int i;
+	unsigned i;
 	for( i = 0; i < component->nchildren; i++)
 	{
 		struct starpu_sched_component * c = component->children[i];
@@ -575,25 +675,46 @@ double starpu_sched_component_estimated_load(struct starpu_sched_component * com
 	return sum;
 }
 
-double starpu_sched_component_estimated_end_min(struct starpu_sched_component * component)
+double starpu_sched_component_estimated_end_min_add(struct starpu_sched_component * component, double exp_len)
 {
 	STARPU_ASSERT(component);
 	double min = DBL_MAX;
-	int i;
+	unsigned i;
+	double ends[component->nchildren];
 	for(i = 0; i < component->nchildren; i++)
 	{
-		double tmp = component->children[i]->estimated_end(component->children[i]);
+		double tmp = ends[i] = component->children[i]->estimated_end(component->children[i]);
 		if(tmp < min)
 			min = tmp;
 	}
+	if (exp_len > 0)
+	{
+		/* We don't know which workers will do this, assume it will be
+		 * evenly distributed to existing work */
+		int card = starpu_bitmap_cardinal(component->workers_in_ctx);
+		if (card == 0)
+			/* Oops, no resources to compute our tasks. Let's just hope that
+			 * we will be given one at some point */
+			card = 1;
+		for(i = 0; i < component->nchildren; i++)
+		{
+			exp_len += ends[i] - min;
+		}
+		min += exp_len / card;
+	}
 	return min;
 }
 
+double starpu_sched_component_estimated_end_min(struct starpu_sched_component * component)
+{
+  return starpu_sched_component_estimated_end_min_add(component, 0.);
+}
+
 double starpu_sched_component_estimated_end_average(struct starpu_sched_component * component)
 {
 	STARPU_ASSERT(component);
 	double sum = 0.0;
-	int i;
+	unsigned i;
 	for(i = 0; i < component->nchildren; i++)
 		sum += component->children[i]->estimated_end(component->children[i]);
 	return sum / component->nchildren;
@@ -606,8 +727,7 @@ static void take_component_and_does_nothing(struct starpu_sched_component * comp
 struct starpu_sched_component * starpu_sched_component_create(struct starpu_sched_tree *tree, const char *name)
 {
 	struct starpu_sched_component *component;
-	_STARPU_MALLOC(component, sizeof(*component));
-	memset(component,0,sizeof(*component));
+	_STARPU_CALLOC(component, 1, sizeof(*component));
 	component->tree = tree;
 	component->workers = starpu_bitmap_create();
 	component->workers_in_ctx = starpu_bitmap_create();
@@ -619,7 +739,7 @@ struct starpu_sched_component * starpu_sched_component_create(struct starpu_sche
 	component->can_push = starpu_sched_component_can_push;
 	component->can_pull = starpu_sched_component_can_pull;
 	component->estimated_load = starpu_sched_component_estimated_load;
-	component->estimated_end = starpu_sched_component_estimated_end_average;
+	component->estimated_end = starpu_sched_component_estimated_end_min;
 	component->deinit_data = take_component_and_does_nothing;
 	component->notify_change_workers = take_component_and_does_nothing;
 	component->name = strdup(name);
