@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2013, 2016-2017  Université de Bordeaux
- * Copyright (C) 2012-2014, 2016, 2017  CNRS
- * Copyright (C) 2011-2013  INRIA
+ * Copyright (C) 2011-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -21,32 +19,13 @@
 #include <hwloc.h>
 #include "core/workers.h"
 
-static unsigned tree_has_next(struct starpu_worker_collection *workers, struct starpu_sched_ctx_iterator *it)
+static unsigned tree_has_next_unblocked_worker(struct starpu_worker_collection *workers, struct starpu_sched_ctx_iterator *it)
 {
 	STARPU_ASSERT(it != NULL);
 	if(workers->nworkers == 0)
 		return 0;
 
 	struct starpu_tree *tree = (struct starpu_tree*)workers->collection_private;
-	int *workerids;
-	int nworkers;
-	int w;
-
-	if (it->value) {
-		struct starpu_tree *node = it->value;
-		/* Are there workers left to be processed in the current node? */
-		nworkers = _starpu_bindid_get_workerids(node->id, &workerids);
-		for(w = 0; w < nworkers; w++)
-		{
-			if(!it->visited[workerids[w]] && workers->present[workerids[w]] )
-			{
-				/* Still some! */
-				it->possible_value = node;
-				return 1;
-			}
-		}
-	}
-
 	struct starpu_tree *neighbour = starpu_tree_get_neighbour(tree, (struct starpu_tree*)it->value, it->visited, workers->present);
 
 	if(!neighbour)
@@ -57,14 +36,26 @@ static unsigned tree_has_next(struct starpu_worker_collection *workers, struct s
 		return 0;
 	}
 	int id = -1;
-	nworkers = _starpu_bindid_get_workerids(neighbour->id, &workerids);
+	int *workerids;
+	int nworkers = starpu_bindid_get_workerids(neighbour->id, &workerids);
+	int w;
 	for(w = 0; w < nworkers; w++)
 	{
 		if(!it->visited[workerids[w]] && workers->present[workerids[w]])
 		{
-			id = workerids[w];
-			it->possible_value = neighbour;
-			break;
+			if(workers->is_unblocked[workerids[w]])
+			{
+				id = workerids[w];
+				it->possible_value = neighbour;
+				break;
+			}
+			else
+			{
+				it->visited[workerids[w]] = 1;
+				it->value = neighbour;
+
+				return tree_has_next_unblocked_worker(workers, it);
+			}
 		}
 	}
 
@@ -73,7 +64,7 @@ static unsigned tree_has_next(struct starpu_worker_collection *workers, struct s
 	return 1;
 }
 
-static int tree_get_next(struct starpu_worker_collection *workers, struct starpu_sched_ctx_iterator *it)
+static int tree_get_next_unblocked_worker(struct starpu_worker_collection *workers, struct starpu_sched_ctx_iterator *it)
 {
 	int ret = -1;
 
@@ -91,11 +82,11 @@ static int tree_get_next(struct starpu_worker_collection *workers, struct starpu
 
 
 	int *workerids;
-	int nworkers = _starpu_bindid_get_workerids(neighbour->id, &workerids);
+	int nworkers = starpu_bindid_get_workerids(neighbour->id, &workerids);
 	int w;
 	for(w = 0; w < nworkers; w++)
 	{
-		if(!it->visited[workerids[w]] && workers->present[workerids[w]])
+		if(!it->visited[workerids[w]] && workers->present[workerids[w]] && workers->is_unblocked[workerids[w]])
 		{
 			ret = workerids[w];
 			it->visited[workerids[w]] = 1;
@@ -104,7 +95,6 @@ static int tree_get_next(struct starpu_worker_collection *workers, struct starpu
 		}
 	}
 	STARPU_ASSERT_MSG(ret != -1, "bind id not correct");
-
 	return ret;
 }
 
@@ -126,7 +116,7 @@ static unsigned tree_has_next_master(struct starpu_worker_collection *workers, s
 	}
 	int id = -1;
 	int *workerids;
-	int nworkers = _starpu_bindid_get_workerids(neighbour->id, &workerids);
+	int nworkers = starpu_bindid_get_workerids(neighbour->id, &workerids);
 	int w;
 	for(w = 0; w < nworkers; w++)
 	{
@@ -161,7 +151,7 @@ static int tree_get_next_master(struct starpu_worker_collection *workers, struct
 
 
 	int *workerids;
-	int nworkers = _starpu_bindid_get_workerids(neighbour->id, &workerids);
+	int nworkers = starpu_bindid_get_workerids(neighbour->id, &workerids);
 	int w;
 	for(w = 0; w < nworkers; w++)
 	{
@@ -178,6 +168,103 @@ static int tree_get_next_master(struct starpu_worker_collection *workers, struct
 	return ret;
 }
 
+static unsigned tree_has_next(struct starpu_worker_collection *workers, struct starpu_sched_ctx_iterator *it)
+{
+	if(it->possibly_parallel == 1)
+                return tree_has_next_master(workers, it);
+        else if(it->possibly_parallel == 0)
+                return tree_has_next_unblocked_worker(workers, it);
+
+	STARPU_ASSERT(it != NULL);
+	if(workers->nworkers == 0)
+		return 0;
+
+	struct starpu_tree *tree = (struct starpu_tree*)workers->collection_private;
+	int *workerids;
+	int nworkers;
+	int w;
+
+	if (it->value)
+	{
+		struct starpu_tree *node = it->value;
+		/* Are there workers left to be processed in the current node? */
+		nworkers = starpu_bindid_get_workerids(node->id, &workerids);
+		for(w = 0; w < nworkers; w++)
+		{
+			if(!it->visited[workerids[w]] && workers->present[workerids[w]] )
+			{
+				/* Still some! */
+				it->possible_value = node;
+				return 1;
+			}
+		}
+	}
+
+	struct starpu_tree *neighbour = starpu_tree_get_neighbour(tree, (struct starpu_tree*)it->value, it->visited, workers->present);
+
+	if(!neighbour)
+	{
+		starpu_tree_reset_visited(tree, it->visited);
+		it->value = NULL;
+		it->possible_value = NULL;
+		return 0;
+	}
+	int id = -1;
+	nworkers = starpu_bindid_get_workerids(neighbour->id, &workerids);
+	for(w = 0; w < nworkers; w++)
+	{
+		if(!it->visited[workerids[w]] && workers->present[workerids[w]])
+		{
+			id = workerids[w];
+			it->possible_value = neighbour;
+			break;
+		}
+	}
+
+	STARPU_ASSERT_MSG(id != -1, "bind id (%d) for workerid (%d) not correct", neighbour->id, id);
+
+	return 1;
+}
+
+static int tree_get_next(struct starpu_worker_collection *workers, struct starpu_sched_ctx_iterator *it)
+{
+	if(it->possibly_parallel == 1)
+                return tree_get_next_master(workers, it);
+        else if(it->possibly_parallel == 0)
+                return tree_get_next_unblocked_worker(workers, it);
+
+	int ret = -1;
+
+	struct starpu_tree *tree = (struct starpu_tree *)workers->collection_private;
+	struct starpu_tree *neighbour = NULL;
+	if(it->possible_value)
+	{
+		neighbour = it->possible_value;
+		it->possible_value = NULL;
+	}
+	else
+		neighbour = starpu_tree_get_neighbour(tree, (struct starpu_tree*)it->value, it->visited, workers->present);
+
+	STARPU_ASSERT_MSG(neighbour, "no element anymore");
+
+
+	int *workerids;
+	int nworkers = starpu_bindid_get_workerids(neighbour->id, &workerids);
+	int w;
+	for(w = 0; w < nworkers; w++)
+	{
+		if(!it->visited[workerids[w]] && workers->present[workerids[w]] )
+		{
+			ret = workerids[w];
+			it->visited[workerids[w]] = 1;
+			it->value = neighbour;
+			break;
+		}
+	}
+	STARPU_ASSERT_MSG(ret != -1, "bind id not correct");
+
+	return ret;
+}
 
 static int tree_add(struct starpu_worker_collection *workers, int worker)
 {
@@ -205,6 +292,7 @@ static int tree_remove(struct starpu_worker_collection *workers, int worker)
 				break;
 			}
 		workers->present[worker] = 0;
+		workers->is_unblocked[worker] = 0;
 		workers->is_master[worker] = 0;
 		workers->nworkers--;
 		return worker;
@@ -225,6 +313,7 @@ static void tree_init(struct starpu_worker_collection *workers)
 	{
 		workers->workerids[i] = -1;
 		workers->present[i] = 0;
+		workers->is_unblocked[i] = 0;
 		workers->is_master[i] = 0;
 	}
 
@@ -242,12 +331,28 @@ static void tree_init_iterator(struct starpu_worker_collection *workers, struct
 	(void) workers;
 	it->value = NULL;
 	it->possible_value = NULL;
+	it->possibly_parallel = -1;
+	int nworkers = starpu_worker_get_count();
+	memset(&it->visited, 0, nworkers * sizeof(it->visited[0]));
+}
+
+static void tree_init_iterator_for_parallel_tasks(struct starpu_worker_collection *workers, struct starpu_sched_ctx_iterator *it, struct starpu_task *task)
+{
+	if (_starpu_get_nsched_ctxs() <= 1)
+	{
+		tree_init_iterator(workers, it);
+		return;
+	}
+	tree_init_iterator(workers, it);
+	it->possibly_parallel = task->possibly_parallel;
 	int i;
 	int nworkers = starpu_worker_get_count();
 	for(i = 0; i < nworkers; i++)
 	{
-		workers->is_master[i] = (workers->present[i] && !starpu_worker_is_slave(i));
-		it->visited[i] = 0;
+		workers->is_unblocked[i] = (workers->present[i] && !starpu_worker_is_blocked_in_parallel(i));
+		if(!it->possibly_parallel) /* don't bother filling the table with masters we won't use it anyway */
+			continue;
+		workers->is_master[i] = (workers->present[i] && !starpu_worker_is_blocked_in_parallel(i) && !starpu_worker_is_slave_somewhere(i));
 	}
 }
 
@@ -255,13 +360,12 @@ struct starpu_worker_collection worker_tree =
 {
 	.has_next = tree_has_next,
 	.get_next = tree_get_next,
-	.has_next_master = tree_has_next_master,
-	.get_next_master = tree_get_next_master,
 	.add = tree_add,
 	.remove = tree_remove,
 	.init = tree_init,
 	.deinit = tree_deinit,
 	.init_iterator = tree_init_iterator,
+	.init_iterator_for_parallel_tasks = tree_init_iterator_for_parallel_tasks,
 	.type = STARPU_WORKER_TREE
 };
 
