@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2012, 2014  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013, 2014  CNRS
+ * Copyright (C) 2009-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -22,7 +21,7 @@
 #include <sys/time.h>
 
 #include <starpu.h>
-#include <config.h>
+#include <common/config.h>
 
 #include "starpufft.h"
 #ifdef STARPU_USE_CUDA
@@ -198,6 +197,7 @@ static int can_execute(unsigned workerid, struct starpu_task *task STARPU_ATTRIB
 
 #include "starpufftx1d.c"
 #include "starpufftx2d.c"
+#include "starpufftx3d.c"
 
 struct starpu_task *
 STARPUFFT(start)(STARPUFFT(plan) plan, void *_in, void *_out)
@@ -242,6 +242,17 @@ STARPUFFT(start)(STARPUFFT(plan) plan, void *_in, void *_out)
 			}
 			task = STARPUFFT(start2dC2C)(plan, plan->in_handle, plan->out_handle);
 			break;
+		case 3:
+			starpu_vector_data_register(&plan->in_handle, STARPU_MAIN_RAM, (uintptr_t) plan->in, plan->totsize, sizeof(STARPUFFT(complex)));
+			if (!PARALLEL)
+				starpu_vector_data_register(&plan->out_handle, STARPU_MAIN_RAM, (uintptr_t) plan->out, plan->totsize, sizeof(STARPUFFT(complex)));
+			if (PARALLEL)
+			{
+				for (z = 0; z < plan->totsize1; z++)
+					plan->twist1_tasks[z]->handles[0] = plan->in_handle;
+			}
+			task = STARPUFFT(start3dC2C)(plan, plan->in_handle, plan->out_handle);
+			break;
 		default:
 			STARPU_ABORT();
 			break;
@@ -451,6 +462,8 @@ STARPUFFT(showstats)(FILE *out)
 	for (worker = 0, total = 0; worker < starpu_worker_get_count(); worker++)
 		total += task_per_worker[worker];
 
+	if (!total)
+		return;
 	for (worker = 0; worker < starpu_worker_get_count(); worker++)
 	{
 		if (task_per_worker[worker])
@@ -502,6 +515,20 @@ STARPUFFT(report_error)(const char *func, const char *file, int line, cufftResul
 	case CUFFT_UNALIGNED_DATA:
 		errormsg = "unaligned data";
 		break;
+#if defined(MAX_CUFFT_ERROR) && (MAX_CUFFT_ERROR >= 0xE)
+	case CUFFT_INCOMPLETE_PARAMETER_LIST:
+		errormsg = "incomplete parameter list";
+		break;
+	case CUFFT_INVALID_DEVICE:
+		errormsg = "invalid device";
+		break;
+	case CUFFT_PARSE_ERROR:
+		errormsg = "parse error";
+		break;
+	case CUFFT_NO_WORKSPACE:
+		errormsg = "no workspace";
+		break;
+#endif /* MAX_CUFFT_ERROR >= 0xE */
 	default:
 		errormsg = "unknown error";
 		break;
