@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
- * Copyright (C) 2010-2015                                CNRS
- * Copyright (C) 2009-2012,2014                           Université de Bordeaux
+ * Copyright (C) 2009-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -111,7 +109,7 @@ int main(int argc, char *argv[])
 {
 	int i, ret;
 	int size;
-	int n = 0, m = 0;
+	int n = 0, m = 0, p = 0;
 	STARPUFFT(plan) plan;
 	starpu_data_handle_t in_handle, out_handle;
 #ifdef STARPU_HAVE_FFTW
@@ -126,6 +124,11 @@ int main(int argc, char *argv[])
 	double timing;
 	size_t bytes;
 #endif
+	struct starpu_conf conf;
+	starpu_conf_init(&conf);
+	/* FIXME: the testcase needs to be updated to properly support cuFFT */
+	conf.ncuda = 0;
+	ret = starpu_init(&conf);
 
 	ret = starpu_init(NULL);
 	if (ret == -ENODEV) return 77;
@@ -152,6 +155,15 @@ int main(int argc, char *argv[])
 		/* 2D */
 		size = n * m;
 	}
+	else if (argc == 4)
+	{
+		n = atoi(argv[1]);
+		m = atoi(argv[2]);
+		p = atoi(argv[3]);
+
+		/* 3D */
+		size = n * m * p;
+	}
 	else
 	{
 		assert(0);
@@ -161,10 +173,12 @@ int main(int argc, char *argv[])
 	bytes = size * sizeof(STARPUFFT(complex));
 #endif
 
-	STARPUFFT(complex) *in = STARPUFFT(malloc)(size * sizeof(*in));
+	STARPUFFT(complex) *in_orig = STARPUFFT(malloc)(size * sizeof(*in_orig));
 	starpu_srand48(0);
 	for (i = 0; i < size; i++)
-		in[i] = starpu_drand48() + I * starpu_drand48();
+		in_orig[i] = starpu_drand48() + I * starpu_drand48();
+
+	STARPUFFT(complex) *in = STARPUFFT(malloc)(size * sizeof(*in));
 
 	STARPUFFT(complex) *out = STARPUFFT(malloc)(size * sizeof(*out));
 
@@ -196,6 +210,16 @@ int main(int argc, char *argv[])
 #endif
 #ifdef STARPU_USE_CUDA
 		STARPU_ASSERT(cufftPlan2d(&cuda_plan, n, m, _CUFFT_C2C) == CUFFT_SUCCESS);
+#endif
+	}
+	else if (argc == 4)
+	{
+		plan = STARPUFFT(plan_dft_3d)(n, m, p, SIGN, 0);
+#ifdef STARPU_HAVE_FFTW
+		fftw_plan = _FFTW(plan_dft_3d)(n, m, p, NULL, (void*) 1, SIGN, FFTW_ESTIMATE);
+#endif
+#ifdef STARPU_USE_CUDA
+		STARPU_ASSERT(cufftPlan3d(&cuda_plan, n, m, p, _CUFFT_C2C) == CUFFT_SUCCESS);
 #endif
 	}
 	else
@@ -204,6 +228,7 @@ int main(int argc, char *argv[])
 	}
 
 #ifdef STARPU_HAVE_FFTW
+	memcpy(in, in_orig, size * sizeof(*in));
 	gettimeofday(&begin, NULL);
 	_FFTW(execute_dft)(fftw_plan, in, out_fftw);
 	gettimeofday(&end, NULL);
@@ -212,10 +237,11 @@ int main(int argc, char *argv[])
 	printf("FFTW took %2.2f ms (%2.2f MB/s)\n\n", timing/1000, bytes/timing);
 #endif
 #ifdef STARPU_USE_CUDA
+	memcpy(in, in_orig, size * sizeof(*in));
 	gettimeofday(&begin, NULL);
 	if (cufftExecC2C(cuda_plan, (cufftComplex*) in, (cufftComplex*) out_cuda, CUFFT_FORWARD) != CUFFT_SUCCESS)
 		printf("erf2\n");
-	if ((cures = cudaThreadSynchronize()) != cudaSuccess)
+	if ((cures = cudaDeviceSynchronize()) != cudaSuccess)
 		STARPU_CUDA_REPORT_ERROR(cures);
 	gettimeofday(&end, NULL);
 	cufftDestroy(cuda_plan);
@@ -223,6 +249,7 @@ int main(int argc, char *argv[])
 	printf("CUDA took %2.2f ms (%2.2f MB/s)\n\n", timing/1000, bytes/timing);
 #endif
 
+	memcpy(in, in_orig, size * sizeof(*in));
 	ret = STARPUFFT(execute)(plan, in, out);
 	if (ret == -1) return 77;
 	STARPUFFT(showstats)(stdout);
@@ -235,6 +262,7 @@ int main(int argc, char *argv[])
 #endif
 
 #if 1
+	memcpy(in, in_orig, size * sizeof(*in));
 	starpu_vector_data_register(&in_handle, STARPU_MAIN_RAM, (uintptr_t) in, size, sizeof(*in));
 	starpu_vector_data_register(&out_handle, STARPU_MAIN_RAM, (uintptr_t) out, size, sizeof(*out));
 
@@ -270,6 +298,7 @@ int main(int argc, char *argv[])
 #endif
 #endif
 
+	STARPUFFT(free)(in_orig);
 	STARPUFFT(free)(in);
 	STARPUFFT(free)(out);
 
