@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2013                                Inria
- * Copyright (C) 2012-2013,2015,2017                      CNRS
- * Copyright (C) 2013                                     Université de Bordeaux
+ * Copyright (C) 2011-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -117,10 +115,9 @@ test_matrix_cpu_func(void *buffers[], void *args)
 	}
 }
 
-int
-main(int argc, char **argv)
+int main(int argc, char **argv)
 {
-	data_interface_test_summary *summary;
+	struct data_interface_test_summary summary;
 	struct starpu_conf conf;
 	starpu_conf_init(&conf);
 	conf.ncuda = 2;
@@ -132,17 +129,15 @@ main(int argc, char **argv)
 
 	register_data();
 
-	summary = run_tests(&matrix_config);
-	if (!summary)
-		exit(EXIT_FAILURE);
+	run_tests(&matrix_config, &summary);
 
 	unregister_data();
 
 	starpu_shutdown();
 
-	data_interface_test_summary_print(stderr, summary);
+	data_interface_test_summary_print(stderr, &summary);
 
-	return data_interface_test_summary_success(summary);
+	return data_interface_test_summary_success(&summary);
 
 enodev:
 	return STARPU_TEST_SKIPPED;
