@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011  INRIA
+ * Copyright (C) 2011-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -14,7 +14,6 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
-#include <config.h>
 #include <starpu.h>
 #include "generic.h"
 #include "../../../../helper.h"
@@ -24,6 +23,9 @@ struct stats global_stats;
 #ifdef STARPU_USE_CPU
 void cpu_func(void *buffers[], void *args)
 {
+	(void)buffers;
+	(void)args;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	global_stats.cpu++;
@@ -33,6 +35,9 @@ void cpu_func(void *buffers[], void *args)
 #ifdef STARPU_USE_CUDA
 void cuda_func(void *buffers[], void *args)
 {
+	(void)buffers;
+	(void)args;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	global_stats.cuda++;
@@ -40,6 +45,9 @@ void cuda_func(void *buffers[], void *args)
 
 void cpu_to_cuda_func(void *buffers[], void *args)
 {
+	(void)buffers;
+	(void)args;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	global_stats.cpu_to_cuda++;
@@ -47,6 +55,9 @@ void cpu_to_cuda_func(void *buffers[], void *args)
 
 void cuda_to_cpu_func(void *buffers[], void *args)
 {
+	(void)buffers;
+	(void)args;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	global_stats.cuda_to_cpu++;
@@ -68,6 +79,9 @@ struct starpu_codelet cuda_to_cpu_cl =
 #ifdef STARPU_USE_OPENCL
 void opencl_func(void *buffers[], void *args)
 {
+	(void)buffers;
+	(void)args;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	global_stats.opencl++;
@@ -76,6 +90,9 @@ void opencl_func(void *buffers[], void *args)
 static
 void cpu_to_opencl_func(void *buffers[], void *args)
 {
+	(void)buffers;
+	(void)args;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	global_stats.cpu_to_opencl++;
@@ -84,6 +101,9 @@ void cpu_to_opencl_func(void *buffers[], void *args)
 static
 void opencl_to_cpu_func(void *buffers[], void *args)
 {
+	(void)buffers;
+	(void)args;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	global_stats.opencl_to_cpu++;
@@ -103,8 +123,10 @@ struct starpu_codelet opencl_to_cpu_cl =
 #endif /* !STARPU_USE_OPENCL */
 
 #ifdef STARPU_USE_MIC
-void mic_dummy_kernel(void *buffers[], void *cl_arg)
+void mic_dummy_kernel(void *buffers[], void *args)
 {
+	(void)buffers;
+	(void)args;
 }
 
 starpu_mic_kernel_t mic_get_kernel()
@@ -136,6 +158,9 @@ starpu_mic_kernel_t cpu_to_mic_func()
 
 void mic_to_cpu_func(void *buffers[], void *args)
 {
+	(void)buffers;
+	(void)args;
+
 	STARPU_SKIP_IF_VALGRIND;
 
 	global_stats.mic_to_cpu++;
