@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2015,2017                                Inria
- * Copyright (C) 2016,2017,2019                           CNRS
- * Copyright (C) 2014-2016                                Université de Bordeaux
+ * Copyright (C) 2014-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -83,7 +81,7 @@ int main(int argc, char **argv)
 	starpu_data_handle_t handle;
 	void *allocated;
 
-	setenv("STARPU_LIMIT_CPU_MEM", TOTAL, 1);
+	setenv("STARPU_LIMIT_CPU_NUMA_MEM", TOTAL, 1);
 
 	starpu_conf_init(&conf);
 	conf.ncpus = 1;
