@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2015-2016                                Université de Bordeaux
+ * Copyright (C) 2015-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -27,8 +27,9 @@
 
 static int x, y;
 
-static void prod(void *descr[], void *_args STARPU_ATTRIBUTE_UNUSED)
+static void prod(void *descr[], void *arg)
 {
+	(void)arg;
 	int *v = (int *)STARPU_VARIABLE_GET_PTR(descr[0]);
 
 	*v = 1;
@@ -43,12 +44,14 @@ static struct starpu_codelet cl_prod =
 
 static void callback0(void *callback_arg)
 {
+	(void)callback_arg;
 	STARPU_ASSERT(x==0);
 	STARPU_ASSERT(y==0);
 }
 
 static void callback(void *callback_arg)
 {
+	(void)callback_arg;
 	STARPU_ASSERT(x>=1);
 	STARPU_ASSERT(y>=1);
 }
@@ -60,8 +63,10 @@ static struct starpu_codelet cl_nowhere =
 	.modes = { STARPU_R, STARPU_R },
 };
 
-static void cons(void *descr[], void *_args STARPU_ATTRIBUTE_UNUSED)
+static void cons(void *descr[], void *_args)
 {
+	(void)_args;
+
 	int *v = (int *)STARPU_VARIABLE_GET_PTR(descr[0]);
 
 	STARPU_ASSERT(*v == 1);
@@ -84,6 +89,13 @@ int main(int argc, char **argv)
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
+	if (starpu_memory_nodes_get_numa_count() > 1)
+	{
+		/* FIXME: assumes only one RAM node */
+		starpu_shutdown();
+		return STARPU_TEST_SKIPPED;
+	}
+
 	starpu_variable_data_register(&handle_x, STARPU_MAIN_RAM, (uintptr_t)&x, sizeof(x));
 	starpu_variable_data_register(&handle_y, STARPU_MAIN_RAM, (uintptr_t)&y, sizeof(y));
 
