@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
- * Copyright (C) 2010-2017                                Université de Bordeaux
- * Copyright (C) 2011-2012,2015-2017                      CNRS
+ * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -25,7 +23,7 @@
 
 #if !defined(STARPU_HAVE_SETENV)
 #warning setenv is not defined. Skipping test
-int main(int argc, char **argv)
+int main(void)
 {
 	return STARPU_TEST_SKIPPED;
 }
@@ -63,7 +61,7 @@ static uint64_t get_total_memory_size(void)
 }
 #endif
 
-void dummy_func(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void dummy_func(void *descr[], void *_args)
 {
 }
 
