@@ -1,9 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2015-2019                                CNRS
- * Copyright (C) 2017                                     Inria
- * Copyright (C) 2015-2017,2019                           Université de Bordeaux
- * Copyright (C) 2013                                     Corentin Salingue
+ * Copyright (C) 2015-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013       Corentin Salingue
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -156,15 +154,14 @@ int dotest(struct starpu_disk_ops *ops, char *base, void (*vector_data_register)
 	FPRINTF(stderr, "Testing <%s>\n", text);
 	/* Initialize StarPU without GPU devices to make sure the memory of the GPU devices will not be used */
 	// Ignore environment variables as we want to force the exact number of workers
-	unsetenv("STARPU_NCUDA");
-	unsetenv("STARPU_NOPENCL");
-	unsetenv("STARPU_NMIC");
 	struct starpu_conf conf;
 	int ret = starpu_conf_init(&conf);
 	if (ret == -EINVAL)
 		return EXIT_FAILURE;
+	conf.precedence_over_environment_variables = 1;
 	conf.ncuda = 0;
 	conf.nopencl = 0;
+	conf.nmic = 0;
 	ret = starpu_init(&conf);
 	if (ret == -ENODEV) goto enodev;
 
