@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012-2013                                Inria
- * Copyright (C) 2010-2013,2015,2017                      CNRS
- * Copyright (C) 2013-2016                                Université de Bordeaux
+ * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -23,16 +21,18 @@
  * Test that we support the cpu_func and where deprecated field
  */
 
-void cpu_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void cpu_codelet(void *descr[], void *_args)
 {
+	(void)_args;
 	int *valin = (int *)STARPU_VARIABLE_GET_PTR(descr[0]);
 	int *valout = (int *)STARPU_VARIABLE_GET_PTR(descr[1]);
 
 	*valout = *valin;
 }
 
-void cpu2_codelet(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void cpu2_codelet(void *descr[], void *_args)
 {
+	(void)_args;
 	int *valin = (int *)STARPU_VARIABLE_GET_PTR(descr[0]);
 	int *valout = (int *)STARPU_VARIABLE_GET_PTR(descr[1]);
 
@@ -116,7 +116,7 @@ int submit_codelet(struct starpu_codelet cl, int where)
 	return x != y;
 }
 
-int main(int argc, char **argv)
+int main(void)
 {
 	int ret;
 	unsigned where;
