@@ -1,7 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2017                                CNRS
- * Copyright (C) 2015-2016                                Université de Bordeaux
+ * Copyright (C) 2011-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -29,6 +28,8 @@ void func_cpu(void *descr[], void *_args)
 	int num = STARPU_TASK_GET_NBUFFERS(starpu_task_get_current());
 	int i;
 
+	(void)_args;
+
 	for (i = 0; i < num; i++)
 	{
 		int *x = (int *)STARPU_VARIABLE_GET_PTR(descr[i]);
@@ -130,7 +131,7 @@ int test(int n, struct starpu_codelet *static_codelet)
 	return 0;
 }
 
-int main(int argc, char **argv)
+int main(void)
 {
         int *x;
         int i, ret, loop;
